/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.Serializable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.border.*;

import netscape.ldap.LDAPConnection; // For main()

import util.*;

public class TreePanel extends JPanel
                       implements TreeSelectionListener,
                                  MouseListener,
                                  KeyListener,
                                  Serializable {

    /**
     * Construct Tree using the data model specified.
     *
     * @param model A Directory model
     */
    public TreePanel( IDirModel model ) {
        super();
        _treeRenderer = new TreePanelCellRenderer();
        _model = model;
        setLayout(new BorderLayout());
        setBorder(new EmptyBorder(0, 0, 0, 0));
        Component tree = createTree( model );
        add( tree );
    }

    /**
     * Default constructor for deserialization
     */
    public TreePanel() {
        this( null );
    }


    /**
     *  Returns a Component that contains the tree
     *
     *  @return a Component that contains the tree
     */
    protected Component createTree( TreeModel model ) {
        _tree = new JTree(model);
        // Lines between nodes, as in Windows
        _tree.putClientProperty( "JTree.lineStyle", "Angled" );
        // For now, single selection only
        _tree.getSelectionModel().setSelectionMode(
            TreeSelectionModel.SINGLE_TREE_SELECTION );
        _tree.addFocusListener(new FocusListener() {
            // this causes ALL tree nodes to repaint, which
            // is needed to change colors for selected tree nodes
            public void focusGained(FocusEvent e) {
                JTree tree = (JTree)e.getSource();
                tree.validate();
                tree.repaint();
            }
            public void focusLost(FocusEvent e) {
                JTree tree = (JTree)e.getSource();
                tree.validate();
                tree.repaint();
            }
        });
        // Special cell renderer
        _tree.setCellRenderer(_treeRenderer);
        // Catch all events
        _tree.addTreeSelectionListener(this);
        _tree.addMouseListener(this);
        _tree.addKeyListener(this);

        _treePanel = new JScrollPane();
        _treePanel.getViewport().add(_tree);
        _treePanel.setBorder( new BevelBorder(BevelBorder.LOWERED,
                UIManager.getColor("controlHighlight"),
                UIManager.getColor("control"),
                UIManager.getColor("controlDkShadow"),
                UIManager.getColor("controlShadow")));
        _treePanel.setPreferredSize(new Dimension(200, 200));
        _treePanel.setMinimumSize(new Dimension(1, 1));
        return _treePanel;
    }

    /**
     *  Return tree model
     *
     *  @return the tree model
     */
    public IDirModel getModel() {
        return _model;
    }

    /**
     * Set the model
     *
     * @param model the model
     */
    public void setModel( IDirModel model ) {
        _model = model;
    }

    /**
     * Adds a listener that is interested in receiving
     * DirNodeListener events.
     *
     * @param l An object interested in receiving DirNodeListener
     * events
     */
    public void addDirNodeListener( IDirNodeListener l ) {
        _listenerList.add( IDirNodeListener.class, l );
    }

    /**
     * Removes a listener that is interested in receiving
     * DirNodeListener events.
     *
     * @param l An object interested in receiving DirNodeListener
     * events
     */
    public void removeDirNodeListener( IDirNodeListener l ) {
        _listenerList.remove( IDirNodeListener.class, l );
    }

    /**
     * Dispatch selection events to listeners
     *
     * @param nodes Currently selected nodes
     */
    protected void select( IDirNode[] nodes ) {
        fireSelectionChanged( nodes );
    }

    /**
     * Dispatch "run" event to listeners
     *
     * @param nodes Currently selected nodes
     * @param param Optional additional event info
     */
    protected void run( IDirNode[] nodes, String param ) {
        fireActionInvoked( nodes, DirNodeEvent.RUN, param );
    }

    /**
     * Dispatch selection events to listeners, using the
     * EventListenerList
     *
     * @param nodes Currently selected nodes
     */
    protected void fireSelectionChanged( IDirNode[] nodes ) {
        // Guaranteed to return a non-null array
        Object[] listeners = _listenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for ( int i = listeners.length - 2; i >= 0; i -= 2 ) {
            if ( listeners[i] == IDirNodeListener.class ) {
                IDirNodeListener l =
                    (IDirNodeListener)listeners[i + 1];
                l.selectionChanged( nodes );
            }
        }
    }

    /**
     * Dispatch events to listeners, using the EventListenerList
     *
     * @param nodes Currently selected nodes
     * @param id Identifier of the type of event
     * @param param Optional additional event info
     */
    protected void fireActionInvoked( IDirNode[] nodes,
                                      int id,
                                      String param ) {
        // Guaranteed to return a non-null array
        Object[] listeners = _listenerList.getListenerList();
        DirNodeEvent e = null;
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for ( int i = listeners.length - 2; i >= 0; i -= 2 ) {
            if ( listeners[i] == IDirNodeListener.class ) {
                // Lazily create the event:
                if (e == null)
                    e = new DirNodeEvent( nodes, id, param );
                IDirNodeListener l = (IDirNodeListener)listeners[i + 1];
                l.actionInvoked( e );
            }
        }
    }

    /**
     * Implements TreeSelectionListener. Called when an object
     * is selected in the tree.
     *
     * @param ev Event provided by JTree
     */
    public void valueChanged(TreeSelectionEvent ev) {
        Debug.println( 9, "Tree.valueChanged: " +
                       ev.getPath().getLastPathComponent() );
        IDirNode[] selection = getSelection();
        if ( selection != null ) {
            Debug.println( 9, "Tree.valueChanged: selection = " +
                           selection[0] );
            select( selection );
        }
    }

    /**
     * Implements MouseListener. Called when a mouse button is
     * pressed and released in the tree.
     *
     * @param e Mouse event
     */
    public void mouseClicked(MouseEvent e) {
        IDirNode[] selection = getSelection();
        if (selection != null) {
            if (e.getClickCount() == 2) { // double click
                run( selection, "" );
            }
        }
    }

    /**
     * Implements MouseListener. Called when a mouse button is
     * pressed in the tree.
     *
     * @param e Mouse event
     */
    public void mousePressed(MouseEvent e) {
        if ((_contextMenu != null) && (e.isPopupTrigger())) {
            if (_contextMenu.getComponentCount() > 0) {
                Point p =
                    _treePanel.getViewport().getViewPosition();
                _contextMenu.show(_treePanel, e.getX() - p.x,
                        e.getY() - p.y);
            }
        }
    }

    /**
     * Implements MouseListener.
     *
     * @param e Mouse event
     */
    public void mouseEntered(MouseEvent e) {
    }

    /**
     * Implements MouseListener.
     *
     * @param e Mouse event
     */
    public void mouseExited(MouseEvent e) {
    }

    /**
     * Implements MouseListener. Called when a mouse button is
     * released in the tree.
     *
     * @param e Mouse event
     */
    public void mouseReleased(MouseEvent e) {
        if ((_contextMenu != null) && (e.isPopupTrigger())) {
            if (_contextMenu.getComponentCount() > 0) {
                Point p =
                    _treePanel.getViewport().getViewPosition();
                _contextMenu.show(_treePanel, e.getX() - p.x,
                        e.getY() - p.y);
            }
        }
    }

    /**
     * Implements KeyListener
     *
     * @param e Key event
     */
    public void keyTyped(KeyEvent e) {
    }

    /**
     * Implements KeyListener. Called when a key is pressed.
     *
     * @param e Key event
     */
    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
            IDirNode[] selection = getSelection();
            if (selection != null) {
                run( selection, "" );
            }
        }
    }

    /**
     * Implements KeyListener. Called when a key is released.
     *
     * @param e Key event
     */
    public void keyReleased(KeyEvent e) {
    }

    /**
     * Returns array of selected nodes.
     *
     * @param return array of selected nodes.
     */
    public IDirNode[] getSelection() {
        IDirNode[] selection = null;
        TreePath path[] = _tree.getSelectionPaths();
        if ((path != null) && (path.length > 0)) {
            selection = new IDirNode[path.length];
            for (int index = 0; index < path.length; index++) {
                selection[index] =
                    (IDirNode)path[index].getLastPathComponent();
            }
        }
        return selection;
    }

    /**
     * Make a node selected and visible
     *
     * @param node Node to make visible
     */
    public void setSelectedNode( IDirNode node ) {
        if ( node != null ) {
            TreePath path =
                new TreePath(
                    ((DefaultMutableTreeNode)node).getPath() );
            _tree.expandPath( path );
            _tree.makeVisible( path );
            _tree.scrollPathToVisible( path );
            _tree.repaint();
            _tree.setSelectionPath( path );
        }
    }

    public static void main( String[] args ) {
        JFrame frame = new JFrame("TreePanel");
        frame.addWindowListener( new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Window win = e.getWindow();
                win.setVisible(false);
                win.dispose();
                System.exit(0);
            }
        } );

        try {
            if ( args.length != 4 ) {
                System.err.println( "Usage: TreePanel HOST " +
                                    "PORT " +
                                    "DN PASSWORD" );
                System.exit( 1 );
            }
            LDAPConnection ldc =
                DirUtil.getLDAPConnection(
                    args[0],
                    Integer.parseInt(args[1]),
                    args[2],
                    args[3] );
            DirModel model = new DirModel( ldc );
//          model.setAllowsLeafNodes( true );
            model.initialize( null );
            TreePanel tree = new TreePanel( model );
            JScrollPane sp = new JScrollPane( tree );
            frame.getContentPane().add( sp );
            frame.pack();
            frame.setLocation( 100, 100 );
            frame.show();
        } catch ( Exception e ) {
            System.err.println( e );
            System.exit( 1 );
        }
    }

    protected IDirModel _model;
    protected JTree _tree = null;
    protected TreeCellRenderer _treeRenderer;
    protected JScrollPane _treePanel;
    // Use an EventListenerList to manage different types of
    // listeners
    protected EventListenerList _listenerList =
        new EventListenerList();
    protected JPopupMenu _contextMenu = new JPopupMenu();
}
