/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Netscape Console code.
 *
 * The Initial Developer of the Original Code is Netscape Communications
 * Corporation.  Portions created by Netscape are Copyright (C) 1998
 * Netscape Communications Corporation.  All Rights Reserved.
 */

package util;

import java.io.*;
import java.util.*;

/**
 * The Debug class controls the output of debugging statements. There are
 * two levels of control for the output. First, the output can be prevented
 * outright by disabling the trace flag. Second, the amount of output can
 * be regulated by the trace level. By convention, we use ten levels,
 * designated from 0 to 9. 0 implies least output and 9 implies full output.
 * By default, the trace level is set to 5 and the trace level designated
 * to debugging statements without a specific level is also 5. These
 * statements will be printed when the trace flag is enabled. To reduce the
 * amount of output, debugging statements should be specified with this
 * default level in mind. Note that all statements designated at the
 * current trace level or below will be printed.
 */
public class Debug implements Serializable {
    /**
    * Constructor.
    */
    public Debug() {
        _fTrace = false;
        _iTraceLevel = -1;
    }

    /**
     * Enables or disables debugging.
     * @param fTrace  boolean value
     */
    public static void setTrace(boolean fTrace) {
        _fTrace = fTrace;
        if (!_fTrace)
            _iTraceLevel = -1;
    }

    /**
     * Determine whether debugging is enabled or disabled.
     * @return  boolean value
     */
    public static boolean getTrace() {
        return _fTrace;
    }

    public static boolean isEnabled() {
        return _fTrace;
    }

    /**
     * Set the verbose level of the debugging output.
     * @param iTraceLevel  integer value indicating which debugging statements to display
     */
    public static void setTraceLevel(int iTraceLevel) {
        _iTraceLevel = iTraceLevel;
        if (_iTraceLevel >= 0)
            _fTrace = true;
    }

    /**
     * Retrieves the verbose level of the debugging output.
     * @return  integer value indicating which debugging statements to display
     */
    public static int getTraceLevel() {
        return _iTraceLevel;
    }

    /**
     * Default debug statement indicates trace level of 5.
     * @param s  debug statement
     */
    public static void println(String s) {
        println(5, s);
    }

    /**
     * Default debug statement indicates trace level of 5.
     * @param s  debug statement
     */
    public static void print(String s) {
        print(5, s);
    }

    /**
     * Debug statement specifying at which level to print.
     * @param level  trace level
     * @param s      debug statement
     */
    public static void println(int level, String s) {
        if (_fTrace && _iTraceLevel >= level) {
            System.err.println(s);
        }
    }

    /**
     * Debug statement specifying at which level to print.
     * @param level  trace level
     * @param s      debug statement
     */
    public static void print(int level, String s) {
        if (_fTrace && _iTraceLevel >= level) {
            System.err.print(s);
        }
    }

    /**
     * Default debug statement indicates trace level of 5.
     * @param s  debug statement
     */
    public static void println(int s) {
        println(5, "" + s);
    }

    /**
     * Default debug statement indicates trace level of 5.
     * @param s  debug statement
     */
    public static void print(int s) {
        print(5, "" + s);
    }

    /**
     * Debug statement specifying at which level to print.
     * @param level  trace level
     * @param s      debug statement
     */
    public static void println(int level, int s) {
        println(level, "" + s);
    }

    /**
     * Debug statement specifying at which level to print.
     * @param level  trace level
     * @param s      debug statement
     */
    public static void print(int level, int s) {
        print(level, "" + s);
    }

    private static boolean _fTrace = false; // flag for global tracing
    private static int _iTraceLevel = 5; // default trace level

}
