/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.Enumeration;
import java.util.Hashtable;
import java.io.IOException;
import netscape.ldap.*;

/**
 * Class for retrieving and storing preferences using LDAP. The
 * preferences are stored as a number of individual
 * attributes in a user entry.
 *
 * @version 1.0
 * @author rweltman
 **/
public class LDAPPreferences1 extends Preferences {
    /**
     * Blank constructor for deserialization
     */
    public LDAPPreferences1() {
    }

    /**
     * Normal constructor specifying user ID or some such for
     * identifying the user's preferences, a URL identifying
     * where user entries for preferences are to be found,
     * and a password for the user
     *
     * @param userKey User ID or some other unique identifier
     * @param url LDAP URL indicating the host, port, and base DN
     * from where to search for the user
     * @param password Password for the user
     */
    public LDAPPreferences1( String url, String userKey,
                             String password ) {
        super( userKey );
        try {
            setUrl( url );
        } catch ( java.net.MalformedURLException e ) {
        }
        _password = password;
    }

    /**
     * Set the host, port, and base DN through an LDAP URL
     *
     * @param url LDAP URL indicating the host, port, and base DN
     * from where to search for the user
     */
    public void setUrl( String url )
        throws java.net.MalformedURLException {
        _url = null;
        try {
            LDAPUrl inUrl = new LDAPUrl( url );
            // Make a complete LDAPUrl, including a search
            // filter for uid matching userKey
            String[] attrs = { "dn" }; // No attributes
            _searchUrl = new LDAPUrl( inUrl.getHost(),
                                      inUrl.getPort(),
                                      inUrl.getDN(), attrs,
                                      LDAPConnection.SCOPE_SUB,
                                      "uid=" + getUserKey() );
        } catch ( java.net.MalformedURLException e ) {
            System.err.println( "Incorrect URL syntax: " + url );
            throw e;
        }
    }

    /**
     * Report the host, port, and base DN through an LDAP URL
     *
     * @return LDAP URL indicating the host, port, and base DN
     * from where to search for the user
     */
    public String getUrl() {
        return (_url != null) ? _url.getUrl() : null;
    }

    /**
     * Save any changed preferences
     *
     * @return <CODE>true</CODE> on success
     */
    public boolean save() {
        try {
            Enumeration keys = getChanges().keys();
            // Any changes to save?
            if( !keys.hasMoreElements() ) {
                return true;
            }
            LDAPConnection ldc = new LDAPConnection();
            ldc.connect( _url.getHost(), _url.getPort() );
            ldc.authenticate( _url.getDN(), _password );
            LDAPModificationSet mods = new LDAPModificationSet();
            // Do we need to add the preferences objectclass?
            if ( !_hasOc ) {
                mods.add( LDAPModification.ADD,
                          new LDAPAttribute( "objectclass",
                                             PREFERENCES_OC ) );
            }
            while( keys.hasMoreElements() ) {
                String name = (String)keys.nextElement();
                byte[] value = (byte[])getChanges().get( name );
                LDAPAttribute attr =
                    new LDAPAttribute( name, value );
                mods.add( LDAPModification.REPLACE, attr );
            }
            System.out.println( "LDAPPreferences.save: " + mods );
            ldc.modify( _url.getDN(), mods );
            _hasOc = true;
            getChanges().clear();
            return true;
        } catch ( LDAPException e ) {
            System.err.println( "LDAPPreferences.save: " + e );
        }
        return false;
    }

    /**
     * Read preferences from directory
     *
     * @return true if successful
     */
    protected boolean initialize() {
        clear();
        setInitialized( false );

        if ( _searchUrl == null ) {
            System.err.println( "Incorrect URL syntax" );
            return false;
        }

        // Search for a single matching entry
        if ( _url == null ) {
            String dn = searchEntry();
            if ( dn != null ) {
                // Found it
                String[] attrs = null; // Inefficient...
                // URL to read one entry
                _url = new LDAPUrl( _searchUrl.getHost(),
                                    _searchUrl.getPort(),
                                    dn, attrs,
                                    LDAPConnection.SCOPE_BASE,
                                    "objectclass=*" );
                System.out.println( "URL: " + _url.getUrl() );
                if ( !initializeSchema() ) {
                    return false;
                }
            }
        }
 
        // Now we know the entry containing the preferences
        boolean success = readEntry();
        if ( success ) {
            setInitialized( true );
        } else {
            clear();
        }

        return success;
    }

    /**
     * Get the preference values from an entry and store them
     * internally
     *
     * @param entry An LDAP entry containing preference attributes
     */
    protected void addPropertiesFromEntry( LDAPEntry entry ) {
        // Get the attributes
        Enumeration en = entry.getAttributeSet().getAttributes();
        while ( en.hasMoreElements() ) {
            LDAPAttribute attr = (LDAPAttribute)en.nextElement();
            String attrName = attr.getName().toLowerCase();
            if ( _allowed.contains( attrName ) ) {
                // Assume only one value each
                Enumeration valsEn = attr.getByteValues();
                // Get the values
                if ( valsEn.hasMoreElements() ) {
                    byte[] value = (byte[])valsEn.nextElement();
                    put( attrName, value );
                }
            }
        }
    }

    /**
     * Get the preference values from the directory and store
     * them internally
     *
     * @return <CODE>true</CODE> if the values could be read
     */
    protected boolean readEntry() {
        clear();
        try {
            System.out.println( "LDAPPreferences.readEntry: " +
                                "reading " + _url.getUrl() );
            LDAPEntry entry = LDAPConnection.read( _url );
            addPropertiesFromEntry( entry );
            // Check if the entry has the preferences objectclass
            Enumeration en =
                entry.getAttribute( "objectclass" ).
                    getStringValues();
            while( en.hasMoreElements() ) {
                String val = (String)en.nextElement();
                System.out.println( "objectclass: " + val );
                if ( val.equalsIgnoreCase( PREFERENCES_OC ) ) {
                    _hasOc = true;
                    break;
                }
            }
            return true;
        } catch ( LDAPException e ) {
            System.err.println( "LDAPPreferences.readEntry: " + e );
        }
        return false;
    }

    /**
     * Find the entry of this user
     *
     * @return The DN of the user entry, if found and unique,
     * otherwise <CODE>null</CODE>
     */
    protected String searchEntry() {
        clear();
        LDAPSearchResults res = null;
        try {
            System.out.println(
                "LDAPPreferences.searchEntry: " +
                "searching " + _searchUrl.getUrl() );
            res = LDAPConnection.search( _searchUrl );
        } catch ( LDAPException e ) {
            System.err.println(
                "LDAPPreferences.searchEntry: " + e );
            return null;
        }
        String dn = null;
        while ( res.hasMoreElements() ) {
            if ( dn != null ) {
                System.err.println(
                    "LDAPPreferences.searchEntry: " +
                    "ambiguous user ID " + getUserKey() );
                clear();
                return null;
            }
            try {
                LDAPEntry entry = res.next();
                dn = entry.getDN();
                // Found a match
                System.out.println(
                    "LDAPPreferences.searchEntry: found " + dn );
            } catch ( LDAPException e ) {
                System.err.println(
                    "LDAPPreferences.initialize: " + e );
                continue;
            }
        }
        return dn;
    }

    /**
     * Get the names of the attributes which are preferences by
     * reading the directory schema
     *
     * @return <CODE>true</CODE> if the schema could be read
     */
    protected boolean initializeSchema() {
        if ( _allowed == null ) {
            try {
                LDAPSchema schema = new LDAPSchema();
                LDAPConnection ldc = new LDAPConnection();
                ldc.connect( _url.getHost(), _url.getPort() );
                schema.fetchSchema( ldc );
                LDAPObjectClassSchema oc =
                    schema.getObjectClass( PREFERENCES_OC );
                if ( oc == null ) {
                    System.err.println(
                        "LDAPPreferences.initializeSchema: "
                        + "no object class in schema - " +
                        PREFERENCES_OC );
                    return false;
                }
                _allowed = new Hashtable();
                Enumeration e = oc.getRequiredAttributes();
                while( e.hasMoreElements() ) {
                    String name = (String)e.nextElement();
                    if ( !name.equalsIgnoreCase( "objectclass" ) ) {
                        _allowed.put( name.toLowerCase(), name );
                    }
                }
                e = oc.getOptionalAttributes();
                while( e.hasMoreElements() ) {
                    String name = (String)e.nextElement();
                    if ( !name.equalsIgnoreCase( "objectclass" ) ) {
                        _allowed.put( name.toLowerCase(), name );
                    }
                }
            } catch ( LDAPException e ) {
                System.err.println(
                    "LDAPPreferences.initializeSchema: " + e );
                return false;
            }
        }
        return true;
    }

    public static void main( String[] args ) {
        Preferences prefs = new LDAPPreferences1(
            "ldap://localhost:389/o=airius.com",
            "dakers", "integument" );
        prefs.dump();
        prefs.setString( "teexcolor", "red" );
        prefs.dump();
        prefs.save();
        System.exit( 0 );
    }

    public final String PREFERENCES_OC = "TeexPreferences";

    private LDAPUrl _searchUrl = null; // Host, port, base DN
    private LDAPUrl _url = null; // Complete URL of user entry
    private String _password = null; // User password
    private Hashtable _allowed = null; // Preference attributes
    private boolean _hasOc = false;
}
