 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.MissingResourceException;
import java.io.*;

/**
 * Preferences<BR>
 * Abstract base class for retrieving and storing preferences
 *
 * @version 1.0
 * @author rweltman
 **/
public abstract class Preferences extends Hashtable
                                  implements Serializable {
    /**
     * Blank constructor for deserialization
     */
    public Preferences() {
    }

    /**
     * Normal constructor specifying user ID or some such for
     * identifying the user's preferences
     *
     * @param userKey User ID or some other unique identifier
     */
    public Preferences( String userKey ) {
        setUserKey( userKey );
    }

    /**
     * Report the user ID or other key to the user's preferences
     *
     * @return User ID
     */
    public String getUserKey() {
        return _userKey;
    }

    /**
     * Set the user ID or other key to user preferences
     *
     * @param userKey user ID
     */
    public void setUserKey( String userKey ) {
        _userKey = userKey;
    }

    /**
     * Retrieve a single string-valued preference value
     *
     * @param key Key for the particular preference
     * @return The value of the preference
     * @exception MissingResourceException if not found
     */
    public String getString( String key ) {
        byte[] val = getBytes( key );
        String s = null;
        if ( val != null ) {
            try {
                s = new String( val, "UTF8" );
            } catch ( UnsupportedEncodingException e ) {
            }
        }
        return s;
    }

    /**
     * Retrieve a single raw preference value, just as stored in the
     * directory
     *
     * @param key Key for the particular preference
     * @return The value of the preference
     * @exception MissingResourceException if not found
     */
    public byte[] getBytes( String key ) {
        if ( !isInitialized() ) {
            if ( !initialize() ) {
                return null;
            }
        }
        byte[] value = (byte[])get( key.toLowerCase() );
        if ( value != null ) {
            return value;
        }
        throw new MissingResourceException(
            "Missing value", "Preferences", key );
    }

    /**
     * Retrieve a single preference value, instantiated as a Java class
     *
     * @param key Key for the particular preference
     * @return A deserialized object
     * @exception MissingResourceException if not found
     * @exception IOException on failure to deserialize
     */
    public Object getObject( String key ) throws IOException {
        byte[] val = getBytes( key );
        return deserializeObject( val );
    }

    /**
     * Retrieve a single preference value of integer type
     *
     * @param key Key for the particular preference
     * @return Value of the preference
     * @exception MissingResourceException if not found
     */
    public int getInteger( String key )  {
        String s = getString( key );
        if ( s != null ) {
            return Integer.valueOf(s).intValue();
        }
        // Will never get this far, because there will be a
        // MissingResourceException or NumberFormatException
        // on error
        return -1;
    }

    /**
     * Retrieve a single preference value of boolean type
     *
     * @param key Key for the particular preference
     * @return Value of the preference
     * @throw MissingResourceException if not found
     */
    public boolean getBoolean( String key )  {
        String s = getString( key );
        if ( s != null ) {
            return Boolean.valueOf( s ).booleanValue();
        }
        // Will never get this far, because there will be a
        // MissingResourceException on error
        return false;
    }

    /**
     * Set a single string-valued preference
     *
     * @param key Key for the particular preference
     * @param value Value for the particular preference
     */
    public void setString( String key, String value ) {
        setBytes( key, value.getBytes() );
    }

    /**
     * Set a single raw preference value
     *
     * @param key Key for the particular preference
     * @param value Value for the particular preference
     */
    public void setBytes( String key, byte[] value ) {
        key = key.toLowerCase();
        byte[] old = (byte[])get( key );
        boolean different = ( (old == null) ||
                              (old.length != value.length) );
        int i = value.length - 1;
        while( (i >= 0) && !different ) {
            different = (value[i] != old[i]);
            i--;
        }
        if ( different ) {
            put( key, value );
            _changed.put( key, value );
        }
    }

    /**
     * Set a single integer-valued preference
     *
     * @param key Key for the particular preference
     * @param value Value for the particular preference
     */
    public void setInteger( String key, int value ) {
        String s = Integer.toString( value );
        setString( key, s );
    }

    /**
     * Set a single boolean-valued preference
     *
     * @param key Key for the particular preference
     * @param value Value for the particular preference
     */
    public void setBoolean( String key, boolean value ) {
        String s = new Boolean( value ).toString();
        setString( key, s );
    }

    /**
     * Serialize an object
     *
     * @param key Key for the particular preference
     * @param value Object to serialize
     * @throw IOException on failure to serialize
     */
    public void setObject( String key, Object value )
        throws IOException {
        byte[] val = serializeObject( value );
        setBytes( key, val );
    }

    /**
     * Save any changed preferences
     *
     * @return <CODE>true</CODE> on success
     */
    abstract public boolean save();

    /**
     * Get the hashtable containing attributes which have changed
     *
     * @return The hashtable of changed attributes
     */
    protected Hashtable getChanges() {
        return _changed;
    }

    /**
     * Debugging utility method: dump the contents of the preferences.
     *
     */
    public void dump() {
        if ( !isInitialized() ) {
            initialize();
        }
        Enumeration e = keys();
        while( e.hasMoreElements() ) {
            String s = (String)e.nextElement();
            System.out.println( "  " + s + ": " + getString( s ) );
        }
    }

    /**
     * Initialize the preferences (e.g. from persistent storage)
     *
     * @return <CODE>true</CODE> if the preferences were
     * successfully initialized
     */
    abstract protected boolean initialize();

    /**
     * Report if the preferences have been initialized (e.g. from
     * persistent storage)
     *
     * @return <CODE>true</CODE> if the preferences have been
     * initialized
     */
    protected boolean isInitialized() {
        return _isInitialized;
    }

    /**
     * Set the internal state flag for if the preferences have
     * been initialized (e.g. from persistent storage) or not
     *
     * @param initialized <CODE>true</CODE> if the preferences
     * have been initialized
     */
    protected void setInitialized( boolean initialized ) {
        _isInitialized = initialized;
    }

    /**
     * Deserializes an object - creates an object from a byte array
     *
     * @param byteBuf Serialized form of object
     * @return Deserialized object
     * @throw IOException on failure to deserialize
     */
    protected static Object deserializeObject( byte[] byteBuf )
        throws IOException {
        
        ByteArrayInputStream bis = null;
        ObjectInputStream  objis = null;

        try {
            bis = new ByteArrayInputStream(byteBuf);
            objis = new ObjectInputStream(bis);
            return objis.readObject();
        } catch( Exception ex ) {
            throw( new IOException("Failed to deserialize object") );
        } finally {
            try {
                if (objis != null) {
                    objis.close();
                }
                if (bis != null) {
                    bis.close();
                }
            }
            catch (Exception ex) {}
        }
    }

    /**
     * Serializes an object - converts it to a byte array
     *
     * @param obj Object to serialize
     * @return Serialized form of object
     * @throw IOException on failure to serialize
     */
    protected static byte[] serializeObject( Object obj )
        throws IOException {

        ByteArrayOutputStream bos = null;
        ObjectOutputStream objos = null;

        try {
            bos = new ByteArrayOutputStream();
            objos = new ObjectOutputStream(bos);
            objos.writeObject(obj);
            objos.flush();
            return bos.toByteArray();
        } catch( Exception ex ) {
            throw( new IOException(
                "Failed to serialize object " + obj ) );
        } finally {
            try {
                if (objos != null) {
                    objos.close();
                }
                if (bos != null) {
                    bos.close();
                }
            }
            catch (Exception ex) {}
        }
    }   

    private String _userKey = null;
    private boolean _isInitialized = false;
    private Hashtable _changed = new Hashtable();
}

