import javax.swing.ImageIcon;

import java.net.URL;
import java.net.MalformedURLException;
import java.awt.Toolkit;
import java.awt.Image;
import java.io.InputStream;
import java.io.IOException;
import java.util.Hashtable;

public class RemoteImage extends ImageIcon {
    /*
	 * RemoteImage caching works as follows:
	 *   Whenever a RemoteImage is created from scratch, the path is
	 *   looked up in the class cache to see if the image has already
	 *   been created. If so, the Image from that RemoteImage is
	 *   used to initialize the Image for the new RemoteImage,
	 *   obviating the need to do an expensive (memory wise)
	 *   getSystemImage() operation.
	 */
    static private Hashtable _imageCache = new Hashtable();

    static class StackLookup extends SecurityManager {
        public ClassLoader getLoader() {
           Class[] stack = getClassContext();
 
           // magic number 3 is the class which called RemoteImage, and
           // we want its loader.
           Class c = stack[3];
           return ((c == null) ? null : c.getClassLoader());
        }
    }

    public RemoteImage(String path) {
	    RemoteImage ri;
	    if ((ri = (RemoteImage)_imageCache.get(path)) != null) {
		    // If the image already exists in the cache, use that
		    // image to set the image for this RemoteImage and return.
		    //Debug.println("RemoteImage: reusing image: " + path);
		    setImage(ri.getImage());
			return;
		}

        Image img;

        // First try to find the image as a system resource.
        // This will search the CLASSPATH or codebase, checking
        // archives if found.

        if ((img = getSystemImage(path)) != null) {
            setImage(img);
            setDescription(path);
		    //Debug.println("RemoteImage: caching image: " + path);
			_imageCache.put(path, this);
            return;
        }

        // If we're running as an application, try a relative path
        // from the current working directory. This will throw an
        // exception if the file is not found.
        setImage(Toolkit.getDefaultToolkit().getImage(path));
        setDescription(path);
		_imageCache.put(path, this);
        return;
	}

    public RemoteImage() { }

    public RemoteImage(Image image) { setImage(image); }

    protected Image getSystemImage(String path) {
        InputStream is;
      
        try {
            ClassLoader cl = new StackLookup().getLoader();

			if (cl != null) {
				if ((is = cl.getResourceAsStream(path)) == null)
					return null;
            } else {
				if ((is = ClassLoader.getSystemResourceAsStream(path)) == null)
					return null;
			}
         
            byte[] buf = new byte[is.available()];
            is.read(buf);
            return Toolkit.getDefaultToolkit().createImage(buf);
        } catch (IOException ioe) {
            System.out.println("RemoteImage: " + ioe + "(" + path + ")");
            return null;
        } catch (Exception e) {
            System.out.println("RemoteImage: " + e + "(" + path + ")");
            return null;
        }
    }
}
