/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;
import netscape.ldap.*;
import netscape.ldap.controls.*;
import util.Debug;

/**
 * EntryListAdapter
 * Adapter to hook up a Tree with a SimpleTable, to supply the
 * latter with LDAP data - one-level children of a DN selected
 * in the Tree
 *
 * @version 1.0
 * @author rweltman
 **/
public class EntryListAdapter
             implements IDirNodeListener,Serializable {

    /**
     * Constructor for deserialization
     */
    public EntryListAdapter() {
    }

    /**
     * Constructor for use by clients
     *
     * @param ldc Connection to LDAP server
     * @param table A SimpleTable JavaBean
     */
    public EntryListAdapter( LDAPConnection ldc,
                             SimpleTable table ) {
        _ldc = ldc;
        _table = table;
    }

    /**
     * Set the connection to a server
     *
     * @param ldc Active connection to a server
     */
    public void setLDAPConnection( LDAPConnection ldc ) {
        _ldc = ldc;
    }

    /**
     * Gets the connection to a server
     *
     * @return Active connection to a server
     */
    public LDAPConnection getLDAPConnection() {
        return _ldc;
    }

    /**
     * Sets the table to be updated
     *
     * @param table Table to be kept updated
     */
    public void setTable( SimpleTable table ) {
        _table = table;
    }

    /**
     * Gets the table to be updated
     *
     * @return Table to be kept updated
     */
    public SimpleTable getTable() {
        return _table;
    }

    /**
     * Set the headings to display on columns. The default is:
     * { "Name", "Email", "User ID", "Phone" }
     *
     * @param names Headings to display on columns
     */
    public void setColumnNames( String[] names ) {
        _names = names;
        _table.setColumnNames( _names );
    }

    /**
     * Headings to display on columns. The default is:
     * { "Name", "Email", "User ID", "Phone" }
     *
     * @return Headings to display on columns
     */
    public String[] getColumnNames() {
        return _names;
    }

    /**
     * Get widths of columns. The default is:
     * { 140, 140, 60, 100 }
     *
     * @return Widths of columns
     */
    public int[] getColumnWidths() {
        return _widths;
    }

    /**
     * Set widths of columns. The default is:
     * { 140, 140, 60, 100 }
     *
     * @param widths Widths of columns
     */
    public void setColumnWidths( int[] widths ) {
        _widths = widths;
        _table.setColumnWidths( _widths );
    }

    /**
     * Get attributes to search for. The default is: 
     * { "cn", "mail", "uid", "telephonenumber", "ou",
     *   "o", "displayName" };
     *
     * @return Attributes to search for
     */
    public String[] getAttributesToFetch() {
        return _attrs;
    }

    /**
     * Set attributes to search for. The default is: 
     * { "cn", "mail", "uid", "telephonenumber", "ou",
     *   "o", "displayName" };
     *
     * @param attrs  Attributes to search for
     */
    public void setAttributesToFetch( String[] attrs ) {
        _attrs = attrs;
    }

    /**
     * Get attributes to display. The default is:<BR>
     *<BR>
     * "cn", "mail", "uid", "telephonenumber"
     *
     * @return Attributes to display
     */
    public String[] getAttributesToDisplay() {
        return _displayAttrs;
    }

    /**
     * Set attributes to display. The default is:<BR>
     *<BR>
     * "cn", "mail", "uid", "telephonenumber"
     *
     * @param attrs Attributes to display
     */
    public void setAttributesToDisplay( String[] attrs ) {
        _displayAttrs = attrs;
    }

    /**
     * Get attributes which may be used to display in the cn
     * column, in priority order. The default is:<BR>
     * <BR>
     * "displayName", "cn", "ou", "o"
     *
     * @return Attributes which may be used to display in the
     * cn column
     */
    public String[] getNameSynonyms() {
        return _nameAttrs;
    }

    /**
     * Set attributes which may be used to display in the cn
     * column, in priority order. The default is:<BR>
     * <BR>
     * "displayName", "cn", "ou", "o"
     *
     * @param names Attributes which may be used to display in
     * the cn column
     */
    public void setNameSynonyms( String[] names ) {
        _nameAttrs = names;
    }

    /**
     * Get the search filter for returning children.
     * The default is: "objectclass=*", which returns all
     * children.
     *
     * @return Search filter used to fetch children of a node
     */
    public String getChildFilter() {
        return _childFilter;
    }

    /**
     * Set the search filter for returning children.
     * The default is: "objectclass=*", which returns all
     * children.
     *
     * @param childFilter Search filter used to fetch children
     * of a node
     */
    public void setChildFilter( String childFilter ) {
        _childFilter = childFilter;
    }

    /**
     * The selection changed.
     *
     * @param nodes Array of selected tree nodes
     */
    public void selectionChanged( IDirNode[] nodes ) {
        String dn = nodes[0].getDN();
        Debug.println( "EntryListAdapter.selectionChanged: " +
                       dn );
        _table.removeAllRows();
        try {
            // No limit on the number of results to return
            LDAPSearchConstraints cons =
                _ldc.getSearchConstraints();
            cons.setMaxResults( 0 );
            // Request server-side sorting
            LDAPControl[] controls = new LDAPControl[1];
            controls[controls.length-1] = _sortControl;
            cons.setServerControls( controls );
            // One-level search for specified children
            LDAPSearchResults result =
                _ldc.search( dn, _ldc.SCOPE_ONE,
                             getChildFilter(),
                             getAttributesToFetch(),
                             false, cons );
            LDAPEntry entry;
            // Vector for the entire table
            Vector all = new Vector();
            // Vector for each row
            String[] displayAttrs = getAttributesToDisplay();
            String[] nameAttrs = getNameSynonyms();
            while ( result.hasMoreElements() ) {
                Vector row = new Vector();
                entry = result.next();
                for( int i = 0; i < displayAttrs.length; i++ ) {
                    row.addElement( getAttr( entry,
                                             displayAttrs[i],
                                             nameAttrs ) );
                }
                _table.addRow( row, entry.getDN() );
            }
            // Notify the table that the data has changed
            _table.fireTableStructureChanged();
            // Set the current column headers and widths
            // They may change based on the contents of the
            // entry
            _table.setColumnNames( getColumnNames() );
            _table.setColumnWidths( getColumnWidths() );
        } catch (LDAPException e) {
            System.err.println(
                "EntryLister.selectionChanged" +
                " cannot get entry <" + dn + ">" );
        }
    }

    /**
     * Get a requested attribute from an entry. If the
     * attribute is not present, an empty string is returned.
     * For the special case of "cn", a list of synonyms is
     * processed to find one which is available in the entry.
     *
     * @param entry An entry containing attributes
     * @param name The name of the attribute to retrieve
     * @param nameAttrs A list of synonyms for "cn"
     * @return a value for the attribute
     */
    private static String getAttr( LDAPEntry entry,
                                   String name,
                                   String[] nameAttrs ) {
        String value = "        ";
        LDAPAttribute attr = null;
        // For the special cn case, check a number of possible
        // attributes
        if ( name.equals( "cn" ) ) {
            for( int i = 0;
                 (attr == null) && (i < nameAttrs.length); i++ ) {
                attr = entry.getAttribute( nameAttrs[i] );
            }
            if ( attr == null ) {
                // Fall back to using the RDN
                String[] rdns =
                    LDAPDN.explodeDN( entry.getDN(), true );
                if ( (rdns != null) && (rdns[0] != null) ) {
                    Debug.println(
                        "EntryListAdapter.getAttr: " +
                        "returning RDN <" + rdns[0] +
                        "> for cn" );
                    return rdns[0];
                } else {
                    Debug.println(
                        "EntryListAdapter.getAttr: " +
                        "explodeDN failed for <" +
                        entry.getDN() + ">" );
                }
            }
        } else {
            attr = entry.getAttribute( name );
        }
        if ( attr != null ) {
            Enumeration en = attr.getStringValues();
            if ( (en != null) && (en.hasMoreElements()) ) {
                value = (String)en.nextElement();
            }
        }
        Debug.println(
            "EntryListAdapter.getAttr: <" + entry.getDN() +
            "> value of " + name + "is <" +
            value + ">" );
        return value;
    }

    /**
     * An action was invoked using the mouse or keyboard.
     *
     * @param ev Object indicating the type of event.
     */
    public void actionInvoked( DirNodeEvent ev ) {
        // Not doing anything with the event in this class
        System.out.println( "EntryLister.actionInvoked: " +
                            ev.getNodes()[0] );
    }

    private LDAPConnection _ldc = null;
    private SimpleTable _table = null;

    private static final String[] _sortOrder =
        { "sn", "givenname", "cn", "ou", "o" };
    private static LDAPControl _sortControl;

    private String _childFilter = "objectclass=*";
    private String[] _names = { "Name", "Email", "User ID",
                                "Phone" };
    private int[] _widths = { 140, 140, 70, 100 };
    // Attributes to retrieve
    private String[] _attrs = { "cn", "mail", "uid",
                                "telephonenumber",
                               "ou", "o", "displayName" };
    // Options (in order) to use for the Name field
    private String[] _nameAttrs = { "displayName", "cn", "ou",
                                    "o" };
    // Columns to display
    private String[] _displayAttrs = { "cn", "mail", "uid",
                                       "telephonenumber" };

    /**
     * Initialize a sort control for search requests
     */
    static {
        LDAPSortKey[] keys = new LDAPSortKey[_sortOrder.length];
        for( int i = 0; i < _sortOrder.length; i++ ) {
            keys[i] = new LDAPSortKey( _sortOrder[i] );
        }
        _sortControl = new LDAPSortControl( keys, false );
    }
}
