/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.event.EventListenerList;

/**
 * OrgChartPanel
 * 
 * A panel for rendering an org chart, using a tree of
 * DefaultMutableTreeNode objects. The user object of each node
 * is expected to be a Hashtable, containing "dn", "name" and
 * possibly also "title".
 *
 * @version 1.0
 * @author rweltman
 **/
public class OrgChartPanel extends JPanel
                           implements ActionListener,
                                      Serializable {
    public OrgChartPanel() {
        super();
    }

    public OrgChartPanel( DefaultMutableTreeNode root ) {
        this();
        setRootNode( root );
    }

    /**
     * Recursive method to add a reporting subtree to an
     * existing node
     *
     * @param node An entry to add
     * @param parentPanel A horizontal panel to which we will
     * add an item (which may be a whole subtree)
     * @param leafPanel A vertical panel to which the entry
     * @param expanded <CODE>true</CODE> if this node is
     * to be expanded
     * should be added instead if it does not have any reports
     */
    protected void addChild( DefaultMutableTreeNode node,
                             JPanel parentPanel,
                             JPanel leafPanel,
                             boolean expanded ) {
        Hashtable info = (Hashtable)node.getUserObject();
        if ( info == null ) {
            // Must be a virtual root
            for( int i = 0; i < node.getChildCount(); i++ ) {
                Object o = node.getChildAt( i );
                DefaultMutableTreeNode n =
                    (DefaultMutableTreeNode)o;
                addChild( n, this, leafPanel, false );
            }
            return;
        }

        if ( node.getChildCount() > 0 ) {
            // The entry is a manager, so create a vertical
            // panel to list the entry and its direct children
            VerticalPanel verticalPanel =
                createVerticalPanel();
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.fill = gbc.BOTH;
            gbc.gridwidth = gbc.REMAINDER;
            gbc.weightx = 1.0;
            gbc.weighty = 0.0;

            // Create a box for the name and title of the
            // manager
            ExpansionButton box =
                new ExpansionButton( this, expanded );
            box.setLayout( new GridBagLayout() );
            // Let the vertical panel know when the button
            // is pressed
            box.addActionListener( verticalPanel );
            String name = (String)info.get( "name" );
            JButton button = new JButton( name );
            String dn = (String)info.get( "dn" );
            button.setToolTipText( dn );
            Color background = getExpandedBoxColor();
            button.setBackground( background );
            Border emptyBorder =
                BorderFactory.createEmptyBorder( 2,2,2,2 );
            button.setBorder( emptyBorder );
            button.addActionListener( this );
            button.setActionCommand( dn );
            box.add( button, gbc );
            String title = (String)info.get( "title" );
            if ( title != null ) {
                JLabel label = new JLabel( title );
                label.setBackground( background );
                label.setForeground( getManagerTextColor() );
                label.setBorder( emptyBorder );
                // Same font but italics for title
                Font font = label.getFont();
                font = new Font( font.getName(), font.ITALIC,
                                 font.getSize() );
                label.setFont( font );
                label.setHorizontalAlignment( label.CENTER );
                box.add( label, gbc );
            }
            JPanel hPanel = new JPanel();
            hPanel.setLayout(
                new BoxLayout( hPanel, BoxLayout.X_AXIS ) );

            // Add the shell panel for the manager box. We'll
            // add the manager box itself later when we know
            // if there is more than one column under it.
            verticalPanel.add( Box.createVerticalStrut( 4 ), gbc );
            verticalPanel.add( hPanel, gbc );
            verticalPanel.add( Box.createVerticalStrut( 2 ), gbc );
            // Add this column to the parent panel
            parentPanel.add( verticalPanel );
            parentPanel.add( Box.createHorizontalStrut( 2 ) );
            // Create a horizontal panel for any children we
            // may have. Each manager will be a new column under
            // the manager box, but all non-managers will be
            // listed vertically in the leafPanel.
            JPanel horizontalPanel = createHorizontalPanel();
            horizontalPanel.setLayout( new BoxLayout(
                horizontalPanel, BoxLayout.X_AXIS ) );
            horizontalPanel.setVisible( expanded );
            leafPanel = createLeafPanel();
            leafPanel.setLayout( new BoxLayout(
                leafPanel, BoxLayout.Y_AXIS ) );

            // Recurse for children
            boolean hasChildren = false;
            for(int i = 0; i < node.getChildCount(); i++ ) {
                addChild(
                    (DefaultMutableTreeNode)node.getChildAt( i ),
                    horizontalPanel,
                    leafPanel,
                    false );
                hasChildren = true;
            }

            // If there are non-manager direct reports, add them
            // all (in the leafPanel) to the horizontal panel
            if ( leafPanel.getComponentCount() > 0 ) {
                leafPanel.add( Box.createGlue() );
                horizontalPanel.add( leafPanel );
                hasChildren = true;
            }

            // Left-align the manager box if there is only
            // one column under it
            if ( horizontalPanel.getComponentCount() < 2 ) {
                hPanel.add( box );
                hPanel.add( Box.createGlue() );
            } else {
                hPanel.add( Box.createGlue() );
                hPanel.add( box );
                hPanel.add( Box.createGlue() );
            }
            // Left-align the horizontal panel and add it to
            // this manager's column
            horizontalPanel.add(
                Box.createHorizontalStrut( 8 ) );
            horizontalPanel.add( Box.createGlue() );
            verticalPanel.add( horizontalPanel, gbc );
            gbc.weighty = 1.0;
            verticalPanel.add( Box.createGlue(), gbc );

            box.stateChanged( expanded );

        } else {
            // This is not a manager, so just add it to
            // the leafPanel column
            SubordinateButton button =
                new SubordinateButton(
                    (String)info.get( "name" ) );
            button.setBorder(
                BorderFactory.createEmptyBorder( 1,1,1,1 ) );
            button.setForeground( getSubordinateTextColor() );
            String dn = (String)info.get( "dn" );
            button.setToolTipText( dn );
            button.setActionCommand( dn );
            button.addActionListener( this );
            leafPanel.add( button );
        }
    }

    /**
     * Set the root node of the tree to be represented in this
     * panel. The panel is regenerated.
     *
     * @param root The root node of a tree of reporting
     * relationships
     */
    public void setRootNode( DefaultMutableTreeNode root ) {
        _root = root;
        removeAll();
        setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );
        JPanel leafPanel = createLeafPanel();
        leafPanel.setLayout( new BoxLayout(
            leafPanel, BoxLayout.Y_AXIS ) );
        // Recursively add all children
        addChild( root, this, leafPanel, false );
        // Add any free agents
        if ( leafPanel.getComponentCount() > 0 ) {
            leafPanel.add( Box.createGlue() );
            add( leafPanel );
        }
    }

    /**
     * Get the root node of the tree represented in this
     * panel.
     *
     * @return The root node of a tree of reporting
     * relationships
     */
    public DefaultMutableTreeNode getRootNode() {
        return _root;
    }

    /**
     * Gets the color to be used for the manager name and title
     *
     * @return The color to be used for the manager text
     */
    public Color getManagerTextColor() {
        return _managerTextColor;
    }

    /**
     * Sets the color to be used for the manager name and title
     *
     * @param color The color to be used for the manager text
     */
    public void setManagerTextColor( Color color ) {
        _managerTextColor = color;
        updateProperties( this );
    }

    /**
     * Gets the color to be used for non-manager names
     *
     * @return The color to be used for non-manager names
     */
    public Color getSubordinateTextColor() {
        return _subordinateTextColor;
    }

    /**
     * Sets the color to be used for non-manager names
     *
     * @param color The color to be used for non-manager names
     */
    public void setSubordinateTextColor( Color color ) {
        _subordinateTextColor = color;
        updateProperties( this );
    }

    /**
     * Gets the color to be used for manager boxes when expanded
     *
     * @return The color to be used for boxes when expanded
     */
    public Color getExpandedBoxColor() {
        return _expandedBoxColor;
    }

    /**
     * Sets the color to be used for manager boxes when expanded
     *
     * @param color The color to be used for boxes when expanded
     */
    public void setExpandedBoxColor( Color color ) {
        _expandedBoxColor = color;
        updateProperties( this );
    }

    /**
     * Gets the color to be used for manager boxes when collapsed
     *
     * @return The color to be used for boxes when collapsed
     */
    public Color getCollapsedBoxColor() {
        return _collapsedBoxColor;
    }

    /**
     * Sets the color to be used for manager boxes when collapsed
     *
     * @param color The color to be used for boxes when collapsed
     */
    public void setCollapsedBoxColor( Color color ) {
        _collapsedBoxColor = color;
        updateProperties( this );
    }

    /**
     * Gets the color to be used for manager box borders
     *
     * @return The color to be used for box borders
     */
    public Color getBoxBorderColor() {
        return _boxBorderColor;
    }

    /**
     * Sets the color to be used for manager box borders
     *
     * @param color The color to be used for box borders
     */
    public void setBoxBorderColor( Color color ) {
        _boxBorderColor = color;
        updateProperties( this );
    }

    /**
     * Gets the width to be used for manager box borders
     *
     * @return The width to be used for box borders
     */
    public int getBoxBorderWidth() {
        return _boxBorderWidth;
    }

    /**
     * Sets the width to be used for manager box borders
     *
     * @param width The width to be used for box borders
     */
    public void setBoxBorderWidth( int width ) {
        _boxBorderWidth = width;
        updateProperties( this );
    }

    protected void updateProperties( Component comp ) {
        if ( comp instanceof Container ) {
            Container cont = (Container)comp;
            for( int i = 0; i < cont.getComponentCount(); i++ ) {
                updateProperties( cont.getComponent( i ) );
            }
        }
        if ( comp instanceof ExpansionButton ) {
            ExpansionButton button = (ExpansionButton)comp;
            button.setBorder( BorderFactory.createLineBorder(
                getBoxBorderColor(),
                getBoxBorderWidth()) );
            if ( button.isExpanded() ) {
                button.setChildColor( button,
                                      getExpandedBoxColor() );
            } else {
                button.setChildColor( button,
                                      getCollapsedBoxColor() );
            }
        } else if ( comp instanceof SubordinateButton ) {
            SubordinateButton button = (SubordinateButton)comp;
            button.setForeground( getSubordinateTextColor() );
        }
    }

    // Action Listener handling
    public void addActionListener(ActionListener l) {
        _listenerList.add(ActionListener.class, l);
    }

    public void removeActionListener(ActionListener l) {
        listenerList.remove(ActionListener.class, l);
    }

    /**
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     */
    protected void fireActionPerformed( ActionEvent e ) {
        // Guaranteed to return a non-null array
        Object[] listeners = _listenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for ( int i = listeners.length-2; i>=0; i-=2 ) {
            if ( listeners[i] == ActionListener.class ) {
                ActionListener l =
                    (ActionListener)listeners[i+1];
                l.actionPerformed(e);
            }              
        }
    }

    public void actionPerformed( ActionEvent e ) {
        System.out.println( e.getActionCommand() );
        fireActionPerformed( e );
    }

    protected HorizontalPanel createHorizontalPanel() {
        HorizontalPanel panel = new HorizontalPanel();
        if ( _debug ) {
            panel.setBorder(
                BorderFactory.createLineBorder(
                    Color.blue) );
        }
        return panel;
    }

    protected VerticalPanel createVerticalPanel() {
        VerticalPanel panel = new VerticalPanel();
        if ( _debug ) {
            panel.setBorder(
                BorderFactory.createLineBorder(
                    Color.red) );
        }
        return panel;
    }

    protected JPanel createLeafPanel() {
        JPanel panel = new JPanel();
        if ( _debug ) {
            panel.setBorder(
                BorderFactory.createLineBorder(
                    Color.green) );
        }
        return panel;
    }

    // Default colors and border widths; may be changed through
    // property accessors
    private Color _managerTextColor = Color.black;
    private Color _subordinateTextColor = Color.black;
    private Color _expandedBoxColor =
        new Color( 128, 128, 255 );
    private Color _collapsedBoxColor =
        new Color( 128, 128, 196 );
    private Color _boxBorderColor = Color.black;
    private int _boxBorderWidth = 1;
    // The root of the tree
    private DefaultMutableTreeNode _root = null;
    // Helper object to manage listeners
    EventListenerList _listenerList = new EventListenerList();
    // If _debug is true, containers will have colored borders
    static private final boolean _debug = false;
}

/**
 * Interface to propagate events when a user clicks on a
 * button to expand or collapse something
 */
interface ExpansionListener {
    /**
     * Called when the component is expanded or collapsed
     *
     * @param expanded <CODE>true</CODE> if the new state is
     * expanded
     */
    public void stateChanged( boolean expanded );
    public boolean isExpanded();
}

/**
 * The button/panel containing a manager's name and title.
 * Clicking it causes the subordinates to be expanded/
 * collapsed.
 */
class ExpansionButton extends JButton
                      implements ExpansionListener {
    /**
     * Construct a button for expanding a subtree
     *
     * @param parent The owner panel
     * @param expanded <CODE>true</CODE> if the initial state
     * is to be expanded
     */
    public ExpansionButton( OrgChartPanel parent,
                            boolean expanded ) {
        super();
        _parent = parent;
        setBorder( BorderFactory.createLineBorder(
            _parent.getBoxBorderColor(),
            _parent.getBoxBorderWidth()) );
        stateChanged( expanded );
    }

    /**
     * Adjust visuals when the button is expanded or collapsed
     *
     * @param expanded <CODE>true</CODE> if the new state is
     * expanded
     */
    public void stateChanged( boolean expanded ) {
        if ( expanded ) {
            setChildColor( this,
                           _parent.getExpandedBoxColor() );
            setToolTipText( "Click to collapse" );
        } else {
            setChildColor( this, 
                           _parent.getCollapsedBoxColor() );
            setToolTipText( "Click to expand" );
        }
        _expanded = expanded;
    }

    /**
     * Override Container.add() in order to adjust visuals
     *
     * @param c Component to add
     * @return The added component
     */
    public Component add( Component c ) {
        c = super.add( c );
        stateChanged( _expanded );
        return c;
    }

    /**
     * Report if the object is expanded or collapses
     *
     * @return <CODE>true</CODE> if the current state is
     * expanded
     */
    public boolean isExpanded() {
        return _expanded;
    }

    /**
     * Recurse all child components starting at a given point
     * and set them all to a specified background color
     *
     * @param c Starting component
     * @param col New background color
     */
    void setChildColor( Component c, Color col ) {
        if ( c instanceof Container ) {
            Container cont = (Container)c;
            for( int i = 0;
                 i < cont.getComponentCount(); i++ ) {
                setChildColor( cont.getComponent( i ), col );
            }
        }
        c.setBackground( col );
    }

    private OrgChartPanel _parent;
    private boolean _expanded = true;
}

/**
 * Button for a non-manager's name
 */
class SubordinateButton extends JButton {
    public SubordinateButton( String text ) {
        super( text );
    }
}

/**
 * A panel that contains a manager box and then underneath it a
 * horizontal panel containing reporting subtrees. It is
 * expandible/collapsible.
 */
class VerticalPanel extends JPanel implements ActionListener {
    /**
     * Default constructor
     */
    public VerticalPanel() {
        super();
        // Can't get the vertical alignment right with
        // BoxLayout, so using GridBagLayout
        setLayout( new GridBagLayout() );
    }

    public void actionPerformed( ActionEvent e ) {
        // Find the root container that will need invalidation
        if ( _root == null ) {
            _root = this;
            Container c = this;
            while( (c != null) && (c instanceof Container) ) {
                _root = c;
                c = c.getParent();
            }
        }
        boolean expanded = false;
        // Modify the button visuals
        if ( e.getSource() instanceof ExpansionListener ) {
            ExpansionListener l =
                (ExpansionListener)e.getSource();
            expanded = !l.isExpanded();
            l.stateChanged( expanded );
        }
        // Show or hide the child panels
        for( int i = 0; i < getComponentCount(); i++ ) {
            Component c = getComponent( i );
            if ( c instanceof HorizontalPanel ) {
                c.setVisible( expanded );
            }
        }
        // Redo the layout
        _root.invalidate();
        _root.validate();
    }

    private Container _root = null;
}

/**
 * Extend JPanel just to be able to use RTTI on instances
 */
class HorizontalPanel extends JPanel {
}
