/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.event.EventListenerList;
import netscape.ldap.*;
import netscape.ldap.util.DN;

/**
 * OrgChartTree
 * Display a tree of directory contents, organized by
 * reporting relationships, with tooltips
 *
 * @version 1.0
 * @author rweltman
 **/
public class OrgChartTree extends JTree {

    /**
     * Constructor
     *
     * @param root Root node of a tree of DefaultMutableTreeNode
     * @param ldc An active connection to a directory
     */
    public OrgChartTree( DefaultMutableTreeNode root,
                         LDAPConnection ldc ) {
        super( root );
        _ldc = ldc;
        if ( root.getUserObject() == null ) {
            // Must be a virtual root
            setRootVisible( false );
        }
        setCellRenderer( new OrgChartTreeCellRenderer() );
        addMouseListener( new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) { // double click
                    TreePath path =
                        getPathForLocation( e.getX(),
                                            e.getY() );
                    if ( path == null ) {
                        return;
                    }
                    DefaultMutableTreeNode node =
                        (DefaultMutableTreeNode)
                        path.getLastPathComponent();
                    Hashtable ht =
                        (Hashtable)node.getUserObject();
                    String dn = (String)ht.get( "dn" );
                    if ( dn != null ) {
                        JTable table =
                            new PropertyTable( _ldc, dn );
                        Dimension dim =
                            new Dimension( 500, 350 );
                        new SimpleDialog( dn, table, 20, 20,
                                          dim, false ).show();
                    }
                }
            }
        } );
    }

    /**
     * Enable or diable tooltips
     *
     * @param state <CODE>true</CODE> to enable tooltips
     */
    public void enableToolTips( boolean state ) {
        ToolTipManager mgr = ToolTipManager.sharedInstance();
        if ( state ) {
            // Enable tool tips.
            mgr.registerComponent( this );
        } else {
            mgr.unregisterComponent( this );
        }
    }

    /**
     * Create a frame, get the relationship hierarchy from
     * a directory, display the relationship with an OrgChartTree
     *
     * @param args HOST PORT BASE DN PASSWORD
     */
    static public void main ( String[] args ) {
        // Parse arguments
        String host = "localhost";
        int port = 389;
        String base = null;
        String rootDN = null;
        String authDN = null;
        String authPassword = null;
        boolean invalid = false;
        int i = 0;
        while ( i < args.length ) {
            if ( args[i].compareTo("-b") == 0 ) {
                if ( (i + 1) < args.length ) {
                    base = args[i+1];
                    i++;
                }
            } else if (args[i].compareTo("-D" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    authDN = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-w" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    authPassword = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-h" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    host = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-p" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    port = Integer.parseInt(args[i+1]);
                    i++;
                }
            } else if ( args[i].compareTo("-r" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    rootDN = args[i+1];
                    i++;
                }
            } else {
                invalid = true;
            }
            i++;
        }

        if ( invalid || (base == null) ) {
            System.err.println(
                "Usage: OrgChartTree -b BASEDN " +
                "[-D AUTHDN] [-w AUTHPASSWORD] " +
                "[-r ROOTMANAGERDN] [-h HOST] [-p PORT]" );
            System.exit( 1 );
        }

        // Connect to server
        LDAPConnection ldc = new LDAPConnection();
        try {
            ldc.connect( host,
                         port,
                         authDN,
                         authPassword );
        } catch ( LDAPException e ) {
            System.err.println( e );
            System.exit( 1 );
        }

        // Get a tree of users, organized by reporting
        // structure
        DefaultMutableTreeNode root = null;
        try {
            root = ManagementParser.parseTree( ldc, base,
                                               rootDN );
        } catch ( LDAPException e ) {
            System.err.println( e );
            System.exit( 1 );
        }

        // Create a tree component with the nodes found
        OrgChartTree tree = new OrgChartTree( root, ldc );
        tree.expandPath( tree.getPathForRow( 0 ) );
        tree.enableToolTips( true );

        // Create frame
        Dimension dim = new Dimension( 300, 400 );
        new SimpleDialog( "OrgChartTree " + base, tree, 0, 0,
                          dim, true ).show();

        if ( _debug ) {
            ManagementParser.printTree( root );
        }
    }

    static private final boolean _debug = false;
    // Helper object to manage listeners
    private EventListenerList _listenerList = new EventListenerList();
    private LDAPConnection _ldc = null;
}

