/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.plaf.ColorUIResource;

/**
 * Tree cell renderer for showing title under name for
 * nodes that have a title.
 */
public class OrgChartTreeCellRenderer extends JPanel
                                  implements TreeCellRenderer {

    /**
     * Default constructor
     */
    public OrgChartTreeCellRenderer() {
        // Icon and under that a TreeCell
        setLayout( new BoxLayout(this,BoxLayout.X_AXIS) );
        _icon = new JLabel() {    
            public void setBackground( Color color ) {
                if( color instanceof ColorUIResource ) {
                    color = null;
                }
                super.setBackground( color );
            }
        };
        add( _icon );
        add( Box.createHorizontalStrut(4) );
        add( _cell = new TreeCell() );
    }

    /**
     * TreeCellRenderer interface implementation
     */
    public Component getTreeCellRendererComponent(
        JTree tree,
        Object value,
        boolean isSelected, boolean expanded,
        boolean leaf, int row, boolean hasFocus) {
        setEnabled( tree.isEnabled() );
        _cell.setValue( value );
        _cell.setSelected( isSelected );
        _cell.setFocus( hasFocus );
        if ( leaf ) {
            _icon.setIcon(UIManager.getIcon("Tree.leafIcon"));
        } else if ( expanded ) {
            _icon.setIcon(UIManager.getIcon("Tree.openIcon"));
        } else {
            _icon.setIcon(UIManager.getIcon("Tree.closedIcon"));
        }       
        DefaultMutableTreeNode node =
            (DefaultMutableTreeNode)value;
        Hashtable info = (Hashtable)node.getUserObject();
        setToolTipText( (String)info.get( "dn" ) );
        return this;
    }

    /**
     * Calculate the size required as the sum of the icon and
     * the TreeCell
     *
     * @return The combined size of the icon and TreeCell
     */
    public Dimension getPreferredSize() {
        Dimension iconD = _icon.getPreferredSize();
        Dimension cellD = _cell.getPreferredSize();
        int height = Math.max( iconD.height, cellD.height );
        return new Dimension(iconD.width + cellD.width, height);
    }

    /**
     * Override Component.setBackGround to ignore ColorUIResource
     *
     * @param color New background color
     */
    public void setBackground(Color color) {
        if ( color instanceof ColorUIResource ) {
            color = null;
        }
        super.setBackground(color);
    }

    /**
     * Panel containing Name and Title
     */
    class TreeCell extends JPanel {
        TreeCell() {
            setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
            setOpaque(true);
        }
         
        /**
         * Override Component.setBackGround to ignore
         * ColorUIResource
         *
         * @param color New background color
         */
        public void setBackground(Color color) {
            if(color instanceof ColorUIResource) {
                color = null;
            }
            super.setBackground(color);
        }

        /**
         * Override Component.setPreferredSize to check for a
         * null dimension
         *
         * @param d New size
         */
        public void setPreferredSize(Dimension d) {
            if (d != null) {
                _preferredSize = d;
            }
        }
      
        /**
         * Override Component.getPreferredSize to return an
         * internally calculated size
         *
         * @return Preferred size
         */
        public Dimension getPreferredSize() {
            return _preferredSize;
        }

        /**
         * Set the text contents and fonts based on info in the
         * UserObject of the tree node
         *
         * @param o A tree node with a Hashtable as UserObject
         */
        public void setValue(Object o) {
            DefaultMutableTreeNode node =
                (DefaultMutableTreeNode)o;
            Hashtable info = (Hashtable)node.getUserObject();
            int maxWidth = 0;
            int height = 0;
            _lines = 0;
            removeAll();
            JLabel label =
                new JLabel( (String)info.get( "name" ) );
            add( label );
            Dimension dim = label.getPreferredSize();
            height += dim.height;
            if (maxWidth < dim.width) {
                maxWidth = dim.width;
            }
            _lines++;
            // Check if this is a container node
            if ( node.getChildCount() > 0 ) {
                label =
                    new JLabel( (String)info.get( "title" ) );
                add( label );
                dim = label.getPreferredSize();
                height += dim.height;
                if (maxWidth < dim.width) {
                    maxWidth = dim.width;
                }
                // Same font but italics for title
                Font font = label.getFont();
                font = new Font( font.getName(), font.ITALIC,
                                 font.getSize() );
                label.setFont( font );
                _lines++;
            }
            int margin = (_lines > 1) ? 2 : 0;
            setPreferredSize(
                new Dimension(maxWidth + 6 + 2*margin,
                              height + 2*margin));
        }

        /**
         * Set appropriate visuals for selected/unselected
         *
         * @param isSelected <CODE>true<CODE> if the item is
         * selected
         */
        protected void setSelected( boolean isSelected ) {
            Color bColor;
            if (isSelected) {
                bColor =
                    UIManager.getColor("Tree.selectionBackground");
            } else {
                bColor = UIManager.getColor("Tree.textBackground");
            }
            super.setBackground(bColor);
        }

        /**
         * Set the appropriate visuals for if the item has focus
         * or not.
         *
         * @param hasFocus <CODE>true<CODE> if the item has
         * focus
         */
        protected void setFocus( boolean hasFocus ) {
            int lineWidth = 1;
            // Put a black border around the cell if it contains
            // more than one line (i.e. it contains a title)
            if ( _lines > 1 ) {
                setBorder(BorderFactory.createLineBorder(
                    Color.black, lineWidth));
            } else {
                if ( hasFocus ) {
                    Color lineColor =
                        UIManager.getColor(
                            "Tree.selectionBorderColor");
                    setBorder(BorderFactory.createLineBorder(
                        lineColor));
                } else {
                    setBorder(BorderFactory.createEmptyBorder(
                        1,1,1,1));
                }
            }      
        }

        Dimension _preferredSize;
        int _lines = 0;
    }
    protected JLabel _icon;
    protected TreeCell _cell;
}





