/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import netscape.ldap.*;
import java.util.*;

/**
 * Simple search program to experiment with time & size limits
 */
public class SearchWithLimits {

    /**
     * Do a subtree search using a specified filter
     *
     * @parm args host, port, authDN, password, baseDN, filter,
     * timeLimit, sizeLimit
     */
    public static void main( String[] args ) {

        if ( args.length != 8 ) {
            System.out.println( "Usage: java SearchWithLimits " +
                                "<host> <port> " +
                                "<authdn> <password> " +
                                "<baseDN> <filter> " + 
                                "<server time> <size limit>" );
            System.out.println( "Example:" );
            System.out.println( "    java SearchWithLimits " +
                                "localhost 389 " +
                                "\"\" \"\" " +
                                "\"o=airius.com\" " +
                                "\"(|(cn=sam*)(cn=b*))\" " + 
                                "3 10" );
            System.exit(1);
        }

        String host = args[0];
        int port = Integer.parseInt( args[1] );
        String authid = args[2];
        String authpw = args[3];
        String base = args[4];
        String filter = args[5];
        int serverTime = Integer.parseInt( args[6] );
        int maxResults = Integer.parseInt( args[7] );

        // Attributes to return
        String[] ATTRS = {"cn","mail","telephonenumber"};

        LDAPConnection ld = new LDAPConnection();
        LDAPSearchConstraints cons = ld.getSearchConstraints();
        try {
            // Connect to server and authenticate
            ld.connect( host, port, authid, authpw );

            // Set the max results to return, and the timeout
            cons.setMaxResults( maxResults );
            cons.setServerTimeLimit( serverTime );
            LDAPSearchResults res = ld.search( base,
                                               ld.SCOPE_SUB,
                                               filter,
                                               ATTRS,
                                               false,
                                               cons );
 
            // Loop on results until complete
            while ( res.hasMoreElements() ) {
                try {
                    // Next directory entry
                    LDAPEntry entry = res.next();
                    prettyPrint( entry, ATTRS );
                } catch ( LDAPReferralException e ) {
                    // Ignore referrals
                    continue;
                } catch ( LDAPException e ) {
                    int errCode = e.getLDAPResultCode();
                    if (errCode == 
                           LDAPException.SIZE_LIMIT_EXCEEDED) {
                        System.out.println("Size limit exceeded!");
                    } else if (errCode == 
                               LDAPException.TIME_LIMIT_EXCEEDED) {
                        System.out.println("Time limit exceeded!");
                    } else {
                        System.out.println( e.toString() );
                    }
                    continue;
                }
            }
         } catch( LDAPException e ) {
             System.out.println( e.toString() );
         }

         // Done, so disconnect
         if ( (ld != null) && ld.isConnected() ) {
             try {
                 ld.disconnect();
             } catch ( LDAPException e ) {
                 System.out.println( e.toString() );
             }
         }

         System.exit( 0 );
     }

    /**
     * Print names and values of attributes in an entry
     *
     * @param entry Entry containing attributes
     * @param attrs Array of attribute names to display
     */
    public static void prettyPrint( LDAPEntry entry, 
                                    String[] attrs ) {
        System.out.println( "DN: " + entry.getDN() );   

        // Use array to pick attributes. We could have
        // enumerated them all using LDAPEntry.getAttributes
        // but this gives us control of the display order.
        for ( int i = 0; i < attrs.length; i++ ) {
            LDAPAttribute attr =
                entry.getAttribute( attrs[i] );
            if ( attr == null ) {
                System.out.println( attrs[i] +
                                    " not present" );
                continue;
            }
            Enumeration enumVals = attr.getStringValues();
            // Enumerate on values for this attribute
            boolean hasVals = false;
            while ( (enumVals != null) &&
                    enumVals.hasMoreElements() ) {
                String val = (String)enumVals.nextElement();
                System.out.println( attrs[i] + ": " + val );
                hasVals = true;
            }
            if ( !hasVals ) {
                System.out.println( attrs[i] +
                                    " has no values" );
            }
        }
        System.out.println("-------------------");
    }
}
