import netscape.ldap.LDAPControl;
import netscape.ldap.ber.stream.*;

/**
 * Represents an LDAP v3 server control that specifies that you want
 * the server to send email on completion of an add, modify, or delete
 * operation.<BR>
 * The OID for this control is 5.5.5.5.5.1.
 * <P>
 *
 * You can include the control in any add/modify/delete request by constructing
 * an <CODE>LDAPConstraints</CODE> object and calling the
 * <CODE>setServerControls</CODE> method.  You can then pass this
 * <CODE>LDAPConstraints</CODE> object to the
 * request method of an <CODE>LDAPConnection</CODE> object.
 * <P>
 *
 * For example:
 * <PRE>
 * ...
 * LDAPConnection ld = new LDAPConnection();
 * try {
 *     // Connect to server.
 *     ld.connect( 3, hostname, portnumber, "", "" );
 *
 *     // Create a "critical" email request server control using
 *     // the email address "charlie@ldapproducts.com" and a general message.
 *     EmailRequestControl ctrl =
 *                 new EmailRequestControl(
 *                              "charlie@ldapproducts.com",
 *                              "I just removed Phil from the user database",
 *                              true );
 *     // Register the response control so it can be instantiated when returned
 *     // by the server
 *     LDAPControl.register( EmailResponseControl.EMAILRESPONSE,
 *                           EmailResponseControl.class );
 *
 *     // Create constraints to use the control.
 *     LDAPConstraints cons = new LDAPConstraints();
 *     cons.setServerControls( ctrl );
 *
 *     // Send a delete request.
 *     ld.delete( "uid=phil, o=ldapproducts.com" );
 *     EmailResponseControl response = null;
 *     LDAPControl[] c = _ldc.getResponseControls();
 *     if ( c != null ) {
 *         for( int i = 0; i < c.length; i++ ) {
 *             if ( c[i] instanceof EmailResponseControl) {
 *                 response = (EmailResponseControl)c[i];
 *                 break;
 *             }
 *         }
 *     }
 *     if ( response != null ) {
 *         int resultCode = response.getCode();
 *         String resultMessage = response.getMessage();
 *     ...
 *
 *
 *
 *     ModificationEmailRequest ::= SEQUENCE {
 *            emailAddress   OCTET STRING,
 *            emailMessage   OCTET STRING
 *     }
 *
 * </PRE>
 */
public class EmailRequestControl extends LDAPControl {
    public final static String EMAILREQUEST  = "5.5.5.5.5.1";

    /**
     * Constructs an <CODE>EmailRequestControl</CODE> object with an
     * email address and a message to send
     * @param address Email address to send a report to
     * @param message Message to include in report
     * @param critical <CODE>true</CODE> if the LDAP operation should be
     * discarded when the server does not support this control (in other
     * words, this control is critical to the LDAP operation).
     * @see netscape.ldap.LDAPControl
     */
    public EmailRequestControl( String address,
                                String message,
                                boolean critical) {
        super( EMAILREQUEST, critical, null );
        m_value = createSpecification( address, message );
    }

    /**
     * Create a "flattened" BER encoding of the requested contents,
     * and return it as a byte array.
     * @param address Email address to send a report to
     * @param message Message to include in report
     * @return The byte array of encoded data.
     */
    private byte[] createSpecification( String address, String message ) {
        /* A sequence */
        BERSequence ber = new BERSequence();
        /* Add the two parameters as string values */
        ber.addElement( new BEROctetString( address ) );
        ber.addElement( new BEROctetString( message ) );
        /* Suck out the data and return it */
        return flattenBER( ber );
    }
}

