/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */
import java.util.*;
import netscape.ldap.*;

public class PrintSchema {
    /**
     * Basic quicksort algorithm.
     *
     * @param toSort Array of schema elements to sort; it is sorted in place
     * @param low First index to sort
     * @param high Last index to sort
     */
    private static void quicksort( LDAPSchemaElement[] toSort,
                                   int low,
                                   int high ) {
        if ( low >= high )
            return;

        LDAPSchemaElement pivot = toSort[low];
        int slow = low-1, shigh = high+1;

        while ( true ) {
            do
                shigh--;
            while ( toSort[shigh].getName().compareTo( pivot.getName() ) > 0 );
            do
                slow++;
            while ( pivot.getName().compareTo( toSort[slow].getName() ) > 0 );

            if ( slow >= shigh )
                 break;

             LDAPSchemaElement temp = toSort[slow];
             toSort[slow] = toSort[shigh];
             toSort[shigh] = temp;
        }

        quicksort (toSort, low, shigh);
        quicksort (toSort, shigh+1, high);
    }

    /**
     * Basic quicksort algorithm.
     *
     * @param toSort Array of objects to sort; it is sorted in place
     * @param names Array of Strings to sort by; it is sorted in place
     * @param low First index to sort
     * @param high Last index to sort
     */
    private static void quicksort( Object[] toSort,
                                   String[] names,
                                   int low,
                                   int high ) {
        if ( low >= high )
            return;

        String pivot = names[low];
        int slow = low-1, shigh = high+1;

        while ( true ) {
            do
                shigh--;
            while ( names[shigh].compareTo( pivot ) > 0 );
            do
                slow++;
            while ( pivot.compareTo( names[slow] ) > 0 );

            if ( slow >= shigh )
                 break;

             String temp = names[slow];
             names[slow] = names[shigh];
             names[shigh] = temp;
             Object tempO = toSort[slow];
             toSort[slow] = toSort[shigh];
             toSort[shigh] = tempO;
        }

        quicksort (toSort, names, low, shigh);
        quicksort (toSort, names, shigh+1, high);
    }

    /**
     * Sort an array of schema elements
     *
     * @param elements Array of schema elements to sort in place
     */
    private static void sort( LDAPSchemaElement[] elements ) {
        quicksort( elements, 0, elements.length - 1 );
    }

    /**
     * Sort an array of objects, using an array of keys
     *
     * @param elements Array of objects to sort in place
     * @param names Array of strings to use to sort in place
     */
    private static void sort( Object[] elements, String[] names ) {
        quicksort( elements, names, 0, elements.length - 1 );
    }

    /**
     * Process an enumeration of schema elements, sorting them
     * into an array
     *
     * @param en Enumeration of schema elements
     * @return An array of sorted elements
     */
    private static LDAPSchemaElement[] sortElements( Enumeration en ) {
        Vector v = new Vector();
        int i = 0;
        while( en.hasMoreElements() ) {
            v.addElement( en.nextElement() );
        }
        LDAPSchemaElement[] vals = new LDAPSchemaElement[v.size()];
        en = v.elements();
        while( en.hasMoreElements() ) {
            vals[i++] = (LDAPSchemaElement)en.nextElement();
        }
        v.removeAllElements();
        sort( vals );
        return vals;
    }

    /**
     * Print the qualifiers of an object class definition, in a format
     * similar to what is used in slapd.oc.conf
     *
     * @param schema A complete collection of schema definitions
     * @param ocName Name of the object class to print
     * @param level Indentation level
     */
    private static void printOC( LDAPSchema schema,
                                 String ocName,
                                 int level ) {
        LDAPObjectClassSchema oc = schema.getObjectClass( ocName );
        if ( oc == null ) {
            return;
        }

        String tabs = "";
        for( int i = 0; i < level; i++ ) {
            tabs += '\t';
        }
        System.out.println( '\n' + tabs + ocName );
        System.out.println( tabs + '\t' + "OID" );
        System.out.println( tabs + "\t\t" + oc.getID() );
        System.out.println( tabs + '\t' + "Superior" );
        System.out.println( tabs + "\t\t" + oc.getSuperior() );
        System.out.println( tabs + '\t' + "Description" );
        System.out.println( tabs + "\t\t" + oc.getDescription() );
        System.out.println( tabs + '\t' + "Required" );
        Enumeration vals = oc.getRequiredAttributes();
        while( vals.hasMoreElements() ) {
            String s = (String)vals.nextElement();
            System.out.println( tabs + "\t\t" + s );
        }
        System.out.println( tabs + '\t' + "Optional" );
        vals = oc.getOptionalAttributes();
        while( vals.hasMoreElements() ) {
            String s = (String)vals.nextElement();
            System.out.println( tabs + "\t\t" + s );
        }
    }

    /**
     * Get a string representation of an attribute syntax
     *
     * @param attr An attribute schema definition
     * @return A user-friendly string describing the syntax
     */
    private static String getSyntax( LDAPAttributeSchema attr ) {
        int syntax = attr.getSyntax();
        if ( syntax == attr.cis ) {
            return "case-insensitive string";
        } else if ( syntax == attr.binary ) {
            return "binary";
        } else if ( syntax == attr.integer ) {
            return "integer";
        } else if ( syntax == attr.ces ) {
            return "case-exact string";
        } else if ( syntax == attr.telephone ) {
            return "telephone";
        } else if ( syntax == attr.dn ) {
            return "distinguished name";
        } else {
            return attr.getSyntaxString();
        }
    }

    /**
     * Print the qualifiers of an attribute definition, in a format
     * similar to what is used in slapd.at.conf
     *
     * @param attr The attribute schema object
     */
    private static void printAttribute( LDAPAttributeSchema attr ) {
        System.out.println( attr.getName() );
        System.out.println( '\t' + "OID" );
        System.out.println( "\t\t" + attr.getID() );
        System.out.println( '\t' + "Description" );
        System.out.println( "\t\t" + attr.getDescription() );
        System.out.println( '\t' + "Syntax" );
        System.out.println( "\t\t" + getSyntax( attr ) );
        if ( attr.isSingleValued() ) {
            System.out.println( '\t' + "single-valued" );
        } else {
            System.out.println( '\t' + "multi-valued" );
        }
        Enumeration en = attr.getQualifierNames();
        while( en.hasMoreElements() ) {
            String qualifier = (String)en.nextElement();
            String[] values = attr.getQualifier( qualifier );
            for( int i = 0; i < values.length; i++ ) {
                System.out.println( '\t' + qualifier + " = " +
                                    values[i] );
            }
        }
    }

    /**
     * Print the qualifiers of a matching rule definition, in a format
     * similar to what is used in slapd.at.conf
     *
     * @param match The matching rule schema object
     */
    private static void printMatchingRule( LDAPMatchingRuleSchema match ) {
        System.out.println( match.getName() );
        System.out.println( '\t' + "OID" );
        System.out.println( "\t\t" + match.getID() );
        System.out.println( '\t' + "Description" );
        System.out.println( "\t\t" + match.getDescription() );
        System.out.println( '\t' + "Attributes" );
        String[] attrs = match.getAttributes();
        if( attrs != null ) {
            for( int i = 0; i < attrs.length; i++ ) {
                System.out.println( "\t\t" + attrs[i] );
            }
        }
    }

    /**
     * Print all object classes, using indentation to indicate
     * inheritance
     *
     * @param schema A complete collection of schema definitions
     * @param sorted A hashtable of objectclasses to print (one level)
     * @param level Indentation level
     */
    private static void printTree( LDAPSchema schema,
                                   Hashtable sorted,
                                   int level ) {
        Object[] vals = new Object[sorted.size()];
        String[] names = new String[sorted.size()];
        int i = 0;
        Enumeration en = sorted.keys();
        while( en.hasMoreElements() ) {
            String ocName = (String)en.nextElement();
            names[i] = ocName;
            vals[i++] = sorted.get( ocName.toLowerCase() );
        }
        sort( vals, names );
        for( i = 0; i < vals.length; i++ ) {
            String ocName = names[i];
            printOC( schema, ocName, level );
            Object o = vals[i];
            if ( o instanceof Hashtable ) {
                printTree( schema, (Hashtable)o, level+1 );
            }
        }
    }

    /**
     * Find and add object classes which directly inherit from sup
     * to level
     *
     * @param htOC A table containing all object class definitions
     * @level Table to which to add the children
     * @sup Name of superior of children to add
     */
    private static void addChildren( Hashtable htOC,
                                     Hashtable level,
                                     String sup ) {
        Hashtable ht = (Hashtable)htOC.get( sup.toLowerCase() );
        Enumeration en = ht.keys();
        while( en.hasMoreElements() ) {
            String name = (String)en.nextElement();
            Hashtable table = (Hashtable)htOC.get( name.toLowerCase() );
            if ( table != null ) {
                level.put( name.toLowerCase(), table );
                addChildren( htOC, table, name );
            } else {
                level.put( name, ht.get( name.toLowerCase() ) );
            }
        }
    }

    /**
     * Create a Hashtable for all object classes with a common superior,
     * and a Hashtable containing all these Hashtables
     *
     * @param schema A complete set of all schema definitions
     *
     * @return A Hashtable containing other Hashtables with schema
     * definitions
     */
    private static Hashtable sortObjectClasses( LDAPSchema schema ) {
        Hashtable htOC = new Hashtable();
        Enumeration en = schema.getObjectClasses();
        while( en.hasMoreElements() ) {
            // Sort the object classes by parent
            LDAPObjectClassSchema oc = (LDAPObjectClassSchema)en.nextElement();
            Hashtable table = null;
            String sup = oc.getSuperior();
            if ( sup != null ) {
                // Must not be top
                table = (Hashtable)htOC.get( sup.toLowerCase() );
            } else {
                sup = "";
            }
            if( table == null ) {
                table = new Hashtable();
            }
            table.put( oc.getName().toLowerCase(), oc );
            htOC.put( sup.toLowerCase(), table );
        }
        Hashtable tree = new Hashtable();
        // Recursively add children, starting at "top"
        addChildren( htOC, tree, "top" );
        return tree;
    }

    /**
     * Print the usage message
     */
    private static void doUsage() {
        System.err.println( "Usage: netscape.ldap.LDAPSchema " +
                            "[-D AUTHDN -w AUTHPASSWORD] " +
                            "[-a ATTRIBUTE] [-o OBJECTCLASS] " +
                            "[-m MATCHINGRULE] " +
                            "HOST PORT [OBJECTCLASS]" );
    }

    /**
     * Fetch the schema from the LDAP server at the specified
     * host and port, and print out the schema (including descriptions
     * of its object classes, attribute types, and matching rules).
     * The schema is printed in an easily readable format (not the
     * same as the format expected by an LDAP server).  For example,
     * you can enter the following command to print the schema:
     * <PRE>
     * java netscape.ldap.LDAPSchema myhost.mydomain.com 389
     * </PRE>
     * Options are:<BR>
     * -D AUTHDN        Use this DN to authenticate<BR>
     * -w AUTHPASSWORD  Use this password to authenticate<BR>
     * -o OBJECTCLASS   Print the definition of an object class<BR>
     * -a ATTRIBUTE     Print the definition of an attribute<BR>
     * -m MATCHINGRULE  Print the definition of a matching rule<BR>
     *<BR>
     * The default is to print all schema elements.<BR>
     *
     * @param args The host name and the port number of the LDAP server
     * (for example, <CODE>netscape.ldap.LDAPSchema directory.netscape.com
     * 389</CODE>).
     */
    public static void main( String[] args ) {
        
        String host = null;
        int port = -1;
        String authDN = null;
        String authPassword = null;
        Vector attrList = new Vector();
        Vector ocList = new Vector();
        Vector matchList = new Vector();
        for( int i = 0; i < args.length; i++ ) {
            if ( args[i].startsWith( "-" ) ) {
                if ( i > (args.length - 2) ) {
                    doUsage();
                    System.exit( 1 );
                }
                if ( args[i].equals( "-a" ) ) {
                    i++;
                    attrList.addElement( args[i] );
                } else if ( args[i].equals( "-o" ) ) {
                    i++;
                    ocList.addElement( args[i] );
                } else if ( args[i].equals( "-m" ) ) {
                    i++;
                    matchList.addElement( args[i] );
                } else if ( args[i].equals( "-D" ) ) {
                    i++;
                    authDN = args[i];
                } else if ( args[i].equals( "-w" ) ) {
                    i++;
                    authPassword = args[i];
                } else {
                    doUsage();
                    System.exit( 1 );
                }
            } else if ( host == null ) {
                host = args[i];
            } else if ( port == -1 ) {
                port = Integer.parseInt( args[i] );
            } else {
                doUsage();
                System.exit( 1 );
            }
        }
        if ( (host == null) || (port <= 0) ) {
            doUsage();
            System.exit( 1 );
        }

        LDAPConnection ld = new LDAPConnection();
        try {
            // Connect and get schema
            ld.connect( host, port );
            if ( (authDN != null) && (authPassword != null) ) {
                ld.authenticate( authDN, authPassword );
            }
            LDAPSchema schema = new LDAPSchema();
            schema.fetchSchema( ld );
            ld.disconnect();

            // Print any specific schema elements which were requested
            Enumeration en = ocList.elements();
            while( en.hasMoreElements() ) {
                String name = (String)en.nextElement();
                System.out.println( "" );
                printOC( schema, name, 0 );
            }
            en = attrList.elements();
            while( en.hasMoreElements() ) {
                String name = (String)en.nextElement();
                LDAPAttributeSchema attr = schema.getAttribute( name );
                if ( attr == null ) {
                    break;
                }
                System.out.println( "" );
                printAttribute( attr );
            }
            en = matchList.elements();
            while( en.hasMoreElements() ) {
                String name = (String)en.nextElement();
                LDAPMatchingRuleSchema match = schema.getMatchingRule( name );
                if ( match == null ) {
                    break;
                }
                System.out.println( "" );
                printMatchingRule( match );
            }
            // If specific elements were requested, we're done
            if ( (attrList.size() > 0) ||
                 (ocList.size() > 0) ||
                 (matchList.size() > 0) ) {
                System.exit( 0 );
            }

            // Sort the object classes by inheritance, and print them as tree
            Hashtable tree = sortObjectClasses( schema );
            printOC( schema, "top", 0 );
            printTree( schema, tree, 1 );

            // Sort the attributes and print them
            en = schema.getAttributes();
            LDAPSchemaElement[] elements = sortElements( en );
            for( int i = 0; i < elements.length; i++ ) {
                System.out.println( "" );
                printAttribute( (LDAPAttributeSchema)elements[i] );
            }

            // Sort the matching rules and print them
            en = schema.getMatchingRules();
            elements = sortElements( en );
            for( int i = 0; i < elements.length; i++ ) {
                System.out.println( "" );
                printMatchingRule( (LDAPMatchingRuleSchema)elements[i] );
            }

            System.exit( 0 );
        } catch ( LDAPException e ) {
            System.err.println( e );
        }
    }
}
