/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */
import java.awt.BorderLayout;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.io.Serializable;
import java.util.*;
import javax.swing.*;
import netscape.ldap.*;
import netscape.ldap.controls.*;

class VListModel extends AbstractListModel {
    /**
     * All parameters of the model are passed in to the constructor
     *
     * @param ldc A sufficiently authenticated connection to a server
     * @param base The base DN for the search
     * @param scope The search scope
     * @param filter The search filter
     * @param sortAttrs One or more attributes to sort by
     */
    VListModel( LDAPConnection ldc, String base, int scope, String filter,
                String[] sortAttrs ) {
        _base = base;
        _scope = scope;
        _filter = filter;
        _sortAttrs = sortAttrs;
        _ldc = ldc;
        _cons =
            (LDAPSearchConstraints)_ldc.getSearchConstraints().clone();
    }

    /**
     * Called by JList to get virtual list size. The vertical scrollbar is
     * sized using the return value.
     *
     * @return The size of the virtual list
     */
    public int getSize() {
        if ( !_initialized ) {
            _initialized = true;
            _pageControls = new LDAPControl[2];
            // VLV also require a sort control
            LDAPSortKey[] keys = new LDAPSortKey[_sortAttrs.length];
            for( int i = 0; i < keys.length; i++ ) {
                keys[i] = new LDAPSortKey( _sortAttrs[i] );
            }
            _pageControls[0] = new LDAPSortControl( keys, true );
            // Do an initial search to get the virtual list size
            // Keep one page before and two pages after the start
            _beforeCount = _pageSize;
            _afterCount = _pageSize * 2;
            // Create the initial paged results control
            _vlc = new LDAPVirtualListControl( 0, _beforeCount,
                                               _afterCount, 0 );
            _pageControls[1] = _vlc;
            // Specify necessary controls for VLV
            _cons.setServerControls( _pageControls );
            // Bump the max results requested to unlimited
            _cons.setMaxResults( 0 );
            getPage( 0 );
        }
        return _size;
    }

    /**
     * Get a page starting at a specified index (although we may also fetch
     * some preceding entries to our buffer)
     *
     * @param first The index of the first entry required
     * @return <CODE>true</CODE> if entries could be retrieved
     */
    protected boolean getPage( int first ) {
        _vlc.setRange( first, _beforeCount, _afterCount );
        if ( _debug ) {
            System.out.println( "Setting requested range to " +
                                first + ", -" + _beforeCount + ", +" +
                                _afterCount );
        }
        return getPage();
    }

    /**
     * Fill the buffer with entries from a search
     *
     * @return <CODE>true</CODE> if entries could be retrieved
     */
    protected boolean getEntries() {
        // Empty the buffer
        _entries.removeAllElements();
        // Do a search
        try {
            String[] attrs = { _displayAttr };
            LDAPSearchResults res =
                _ldc.search( _base,
                             _scope,
                             _filter,
                             attrs,
                             false,
                             _cons );
            while ( res.hasMoreElements() ) {
                try {
                    LDAPEntry entry = res.next();
                    // Allow the user to specify that the DN should be displayed
                    if ( _displayAttr.equalsIgnoreCase( "dn" ) ) {
                        _entries.addElement( entry.getDN() );
                    } else {
                        LDAPAttribute attr = entry.getAttribute( attrs[0] );
                        if ( attr != null ) {
                            Enumeration en = attr.getStringValues();
                            while( en.hasMoreElements() ) {
                                String name = (String)en.nextElement();
                                _entries.addElement( name );
                            }
                        }
                    }
                } catch ( LDAPException ex ) {
                    System.out.println( ex + ", enumerating" );
                }
            }
        } catch ( LDAPException e ) {
            System.out.println( e + ", searching" );
            return false;
        }
        if ( _debug ) {
            System.out.println( "Returning " + _entries.size() +
                                " entries" );
        }
        return true;
    }

    /**
     * Fetch a buffer
     *
     * @return <CODE>true</CODE> if entries could be retrieved
     */
    protected boolean getPage() {
        // Get the actual entries
        if ( !getEntries() )
            return false;
        
        // Check if we have a control returned
        LDAPControl[] c = _ldc.getResponseControls();
        LDAPVirtualListResponse response = null;
        if ( c != null ) {
            for( int i = 0; i < c.length; i++ ) {
                if ( c[i] instanceof LDAPVirtualListResponse ) {
                    response = (LDAPVirtualListResponse)c[i];
                    break;
                }
            }
        }
        if ( response != null ) {
            _selectedIndex = response.getFirstPosition() - 1;
            _top = Math.max( 0, _selectedIndex - _beforeCount );
            // Now we know the total size of the virtual list box
            _size = response.getContentCount();
            _vlc.setListSize( _size );
            if ( _debug ) {
                System.out.println( "Virtual window: " + _top +
                                    ".." + (_top+_entries.size()-1) +
                                    " of " + _size );
            }
        } else {
            System.out.println( "Null response control" );
        }
        return true;
    }

    /**
     * Called by JList to fetch data to paint a single list item
     *
     * @param index The index of the item to return
     * @return The list item at the requested index
     */
    public Object getElementAt(int index) {
        if ( _debug ) {
            System.out.println( "need entry " + index );
        }
        if ( (index < _top) || (index >= _top + _entries.size()) ) {
            if ( _debug ) {
                System.out.println( "fetching a page starting at " +
                                    index );
            }
            getPage( index );
        }
        int offset = index - _top;
        if ( (offset < 0) || (offset >= _entries.size()) )
            return new String( "No entry at " + index );
        else
            return _entries.elementAt( offset );
    }

    /**
     * Called by an application to scroll the list so some entry is
     * visible. Considers text to be an initial substring of an entry.
     *
     * @param text Text to match
     * @return <CODE>true</CODE> if entries could be provided
     */
    public boolean typedown( String text ) {
        _pageControls[1] = _vlc =
            new LDAPVirtualListControl( text,
                                        _beforeCount,
                                        _afterCount );
        if ( _debug ) {
            System.out.println( "Setting requested start to " +
                                text + ", -" + _beforeCount + ", +" +
                                _afterCount );
        }
        return getPage();
    }
    
    /**
     * Called by application to find out the virtual selected index
     *
     * @return The index of the first entry in the selected window 
     */
    public int getSelectedIndex() {
        return _selectedIndex;
    }

    /**
     * Called by application to find out the top of the buffer
     *
     * @return The index of the first entry in the buffer
     */
    public int getFirstIndex() {
        return _top;
    }

    /**
     * Report the attribute to display
     *
     * @return The attribute to display
     */
    public String getDisplayAttribute() {
        return _displayAttr;
    }

    /**
     * Set the attribute to display
     *
     * @param attr The attribute to display
     */
    public void setDisplayAttribute( String attr ) {
        _displayAttr = attr;
    }

    /**
     * Enable/disable debug printouts
     *
     * @param debug <CODE>true</CODE> to enable debug printouts
     */
    public void setDebug( boolean debug ) {
        _debug = debug;
    }

    /**
     * Report if debug printouts are enabled
     *
     * @return <CODE>true</CODE> if debug printouts are enabled
     */
    public boolean getDebug() {
        return _debug;
    }

    /**
     * Set the number of entries before and after the top one to buffer
     *
     * @param size The number of entries before/after the top one to buffer
     */
    public void setPageSize( int size ) {
        _pageSize = size;
    }

    private Vector _entries = new Vector();
    protected boolean _initialized = false;
    private int _top = 0;
    protected int _beforeCount;
    protected int _afterCount;
    private int _pageSize = 10;
    private int _selectedIndex = 0;
    protected LDAPControl[] _pageControls = null;
    protected LDAPVirtualListControl _vlc = null;
    protected int _size = -1;
    private String _base;
    private String _filter;
    private int _scope;
    private String[] _sortAttrs;
    private String _displayAttr = "cn";
    protected boolean _debug = false;
    private LDAPConnection _ldc;
    LDAPSearchConstraints _cons;
}

