/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */
import java.awt.BorderLayout;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.io.Serializable;
import java.util.*;
import javax.swing.*;
import netscape.ldap.*;
import netscape.ldap.controls.*;

/**
 * A JavaBean that extends JList and contains a list of sorted names from a
 * directory. It is a property change listener for string changes -
 * the list updates itself and scrolls to the first name matching the
 * characters in a supplied string. This is for typedown support.
 */
public class VListPanel extends JList
                        implements PropertyChangeListener,
                                   Serializable {
    public VListPanel() {
        super();
    }

    public VListPanel( LDAPConnection ldc, String base, int scope,
                       String filter, String[] sortAttrs ) {
        super();
        _ldc = ldc;
        _base = base;
        _scope = scope;
        _filter = filter;
        _sortAttrs = sortAttrs;
        initialize();
    }

    /**
     * Create model and UI
     */
    protected void initialize() {
        // We don't want the JList implementation to compute the width 
        // or height of all of the list cells, so we give it a String 
        // that's as big as we'll need for any cell.  It uses this to 
        // compute values for the fixedCellWidth and fixedCellHeight 
        // properties.
        setPrototypeCellValue("12345678901234567890123456789");

        _model = new VListModel( _ldc, _base, _scope,
                                 _filter, _sortAttrs );
        setModel( _model );
        _model.setPageSize( getVisibleRowCount() );
    }

    /**
     * Report the LDAP connection in use
     *
     * @return The LDAP connection in use
     */
    public LDAPConnection getLDAPConnection() {
        return _ldc;
    }

    /**
     * Set the LDAP connection to use to fill the list
     *
     * @param ldc The LDAP connection to use to fill the list
     */
    public void setLDAPConnection( LDAPConnection ldc ) {
        _ldc = ldc;
        if ( readyToGo() ) {
            initialize();
        }
    }

    /**
     * Report the base DN for the search
     *
     * @return The base DN for the search
     */
    public String getBase() {
        return _base;
    }

    /**
     * Set the base DN for the search
     *
     * @param base The base DN for the search
     */
    public void setBase( String base ) {
        _base = base;
        if ( readyToGo() ) {
            initialize();
        }
    }

    /**
     * Report the scope for the search
     *
     * @return The scope for the search
     */
    public int getScope() {
        return _scope;
    }

    /**
     * Set the scope for the search
     *
     * @param scope The scope for the search
     */
    public void setScope( int scope ) {
        _scope = scope;
        if ( readyToGo() ) {
            initialize();
        }
    }

    /**
     * Report the search filter for the search
     *
     * @return The search filter for the search
     */
    public String getFilter() {
        return _filter;
    }

    /**
     * Set the search filter for the search
     *
     * @param filter The search filter for the search
     */
    public void setFilter( String filter ) {
        _filter = filter;
        if ( readyToGo() ) {
            initialize();
        }
    }

    /**
     * Report the attributes to sort on for the search
     *
     * @return The attributes to sort on for the search
     */
    public String[] getSortAttributes() {
        return _sortAttrs;
    }

    /**
     * Set the attributes to sort on for the search
     *
     * @param attrs The attributes to sort on for the search
     */
    public void setSortAttributes( String[] attrs ) {
        _sortAttrs = attrs;
        if ( readyToGo() ) {
            initialize();
        }
    }

    /**
     * Report the attribute to display
     *
     * @return The attribute to display
     */
    public String getDisplayAttribute() {
        return _model.getDisplayAttribute();
    }

    /**
     * Set the attribute to display
     *
     * @param attr The attribute to display
     */
    public void setDisplayAttribute( String attr ) {
        _model.setDisplayAttribute( attr );
    }

    /**
     * Enable/disable debug printouts
     *
     * @param debug <CODE>true</CODE> to enable debug printouts
     */
    public void setDebug( boolean debug ) {
        if ( _model != null ) {
            _model.setDebug( debug );
        }
    }

    /**
     * Report if debug printouts are enabled
     *
     * @return <CODE>true</CODE> if debug printouts are enabled
     */
    public boolean getDebug() {
        if ( _model != null ) {
            return _model.getDebug();
        }
        return false;
    }

    protected boolean readyToGo() {
        return ( (_ldc != null) && (_base != null) &&
                 (_filter != null) && (_sortAttrs != null) );
    }

    /**
     * On a property change event, do typedown to the new string value
     *
     * @param evt An event indicating a changed typedown string
     */
    public void propertyChange( PropertyChangeEvent evt ) {
        if ( evt.getPropertyName().equals( "typedown" ) ) {
            String newVal = "";
            Object obj = (Object)evt.getNewValue();
            if ( (obj != null) && (obj instanceof String) ) {
                newVal = (String)obj;
            }
            doTypedown( newVal );
        }
    }

    /**
     * Tell the model to do typedown, and then scroll the list so the
     * selected index is visible
     */
    protected void doTypedown( String text ) {
        int top = _model.getFirstIndex();
        _model.typedown( text );
        scrollSelectedToTop( top );
    }

    /**
     * Scroll so that the user-selected first index is visible
     *
     * @param oldTop The top index before we make any changes
     */
    protected void scrollSelectedToTop( int oldTop ) {
        int index = _model.getSelectedIndex();
        // If scrolling down, make sure the selected index becomes
        // the topmost one
        if ( _model.getFirstIndex() > oldTop )
            index += getVisibleRowCount() - 1;
        ensureIndexIsVisible( index );
    }

    private LDAPConnection _ldc = null;
    private String _base = null;
    private int _scope = LDAPConnection.SCOPE_SUB;
    private String _filter = null;
    private String[] _sortAttrs = null;
    private String _displayAttr = null;
    private VListModel _model = null;
//}
   
}
