/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import netscape.ldap.*;
import netscape.ldap.util.LDAPFilterDescriptor;
import netscape.ldap.util.*;
import java.util.*;

/**
 * Frame to select filters from a filter file and do searches
 */

public class FilterSearchDialogSort extends Frame {

    /**
     * Launch a frame to do searches using a filter file
     * and sort the results
     *
     * @param args host, port, authDN, password, base, conf
     */
    public static void main( String[] args ) {
        if ( (args.length != 4) &&
             (args.length != 6 ) ) {
            System.out.println( "Usage: java " +
                                "FilterSearchDialogSort " +
                                "<host> <port> " +
                                "<filterfile> <baseDN> " +
                                "[<authdn> <password>]" );
            System.out.println( "Example:" );
            System.out.println( "    java " +
                                "FilterSearchDialogSort " +
                                "localhost 389 " +
                                "filter.conf \"o=airius.com\"" );
            System.exit(1);
        }

        String host = args[0];
        int port = Integer.parseInt( args[1] );
        String conf = args[2];
        String base = args[3];
        String authid = "";
        String authpw = "";
        if ( args.length > 4 ) {
            authid = args[4];
            authpw = args[5];
        }

        Frame f = new FilterSearchDialogSort(
            "Graphical Sorting LDAP Search", host, port,
            authid, authpw, conf, base );
        f.setSize( 430,280 );
        f.show();
    }

    /**
     * Standard Frame constructor, plus connection parameters
     *
     * @param title Window title
     * @param host Host to search
     * @param port Port number of server
     * @param authid DN to authenticate as (may be "")
     * @param authpw Password for authentication (may be "")
     * @param conf Name of filter configuration file
     * @param base Base DN for subtree search
     */
    public FilterSearchDialogSort( String title,
                                   String host, 
                                   int port,
                                   String authdn, 
                                   String authpw,
                                   String conf, 
                                   String base ) {
        super( title );

        this.host = host;
        this.port = port;
        this.authdn = authdn;
        this.authpw = authpw;
        this.conf = conf;
        this.base = base;

        setLayout(null);

        this.addWindowListener(new WindowAdapter() {
            public void windowClosing( WindowEvent e ) {
                // Disconnect from server
                if ( (ld != null) && ld.isConnected() ) {
                    try {
                        ld.disconnect();
                    } catch ( LDAPException le ) {
                        System.out.println( le.toString() );
                    }
                }
                System.exit(0);
            }
        });

        Label lbl1 = new Label( "Search for:" );
        lbl1.setBounds( 10,36,75,26 );
        add( lbl1 );
        srch = new TextField();
        Font font = new Font("Monospaced",Font.PLAIN,12);
        srch.setFont( font );
        srch.setBounds( 90,36,230,26 );
        add( srch );

        searchb = new Button( "Search" );
        searchb.setBounds( 340,36,80,26 );
        add( searchb );

        output = new TextArea(12,3);
        output.setFont( font );
        output.setEditable( false );
        output.setBounds( 10,70,410,200 );
        add( output );

        searchb.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String srchstr = srch.getText();
                // If there is a search string, do a search
                if ( srchstr.length() > 0 ) {
                    searchLDAP( srchstr );
                }
            }
        });


        // Read the filter description file
        // If not found, exit with an error to the console
        try {
            filterDesc =
                new LDAPFilterDescriptor(conf);
        } catch ( Exception e ) {
            System.out.println( "Cannot load " + conf +
                                " file" );
            System.out.println( "Exiting..." );
            System.exit( 1) ;
        }
    }

    protected void searchLDAP( String srchString ) {
        int status = -1;
        String appendmsg = "";
        LDAPSearchResults res = null;
        LDAPFilterList filtlist = null;
        String  srchfilter = "(|";

        // Check if we are connected first
        if ( (ld == null) || (!ld.isConnected()) ) {
            connectServer();
        }

        // Use the filter configuration file to build a
        // search filter
        try {
            filtlist =
                filterDesc.getFilters("search", srchString);

            while ( filtlist.hasMoreElements() ) {
                LDAPFilter fline = filtlist.next();
                // The actual filter is next
                String fstr = fline.getFilter();
                srchfilter += fstr;
            }
            srchfilter += ")";
            if ( srchfilter.length() == 3 ) {
                // No filters found
                return;
            }
            
            // Now do the search
            res = ld.search( base,
                             scope,
                             srchfilter,
                             ATTRS,
                             false );

            //Sort by sn, given and common name in 
            // ascending order
            String[] sortattr = {"sn", "givenname", "cn"};
            boolean[] ascend = {true, true, true};
            res.sort(new LDAPCompareAttrNames(
                                   sortattr,ascend));

            // Display search filter
            String outres = "Filter=" + srchfilter + "\n" +
                "-------------\n";

            // Loop on results building each line
            while ( res.hasMoreElements() ) {
                try {
                    // Next directory entry
                    LDAPEntry entry = res.next();
                    outres += format( entry );
                    status = 0;
                } catch ( LDAPReferralException e ) {
                    // Ignore referrals
                    continue;
                } catch (LDAPException le) {
                    int rc = le.getLDAPResultCode();
                    if ( rc == le.SIZE_LIMIT_EXCEEDED ) {
                        appendmsg =
                            "\nExceeded size limit";
                    } else if ( rc ==
                                le.TIME_LIMIT_EXCEEDED ) {
                        appendmsg =
                            "\nExceeded time limit";
                    } else {
                        appendmsg = le.toString();
                    }
                }
            }

            outres += appendmsg;
            output.setText( outres ); // Display in text area
            
        } catch( Exception e ) {
            System.out.println( "Search error: " +
                                e.toString() );
        }
    }

    protected void connectServer() {
        // Connect to the LDAP server
        if ( (ld == null) || (!ld.isConnected()) ) {
            try {
                ld = new LDAPConnection();
                ld.connect( host, port, authdn, authpw );
            } catch( LDAPException e ) {
                 System.out.println( "Connect error: " +
                                     e.toString() );
                System.exit( 1 );
            }
        }
    }

    /**
     * Format a string with attribute values from an entry,
     * separated by tabs
     *
     * @param entry LDAP entry containing cn, telephonenumber,
     * and mail
     */
    public String format( LDAPEntry entry ) {
        String outstr = "";

        // Get the data - hardcoded attribute names here!
        String name = getValue( entry, "cn" );
        String phone = getValue( entry, "telephonenumber" );
        String email = getValue( entry, "mail" );

        // Limit the full name to 15 characters     
        if ( name.length() > 15 ) {
            name = name.substring( 0, 15 );
        }

        outstr = name + "\t" + phone + "\t" + email + "\n";

        return outstr;
    }

    /**
     * Get first string value of an attribute from an entry
     * or '-' if not present
     *
     * @param entry LDAP entry containing the attribute
     * @param attrName Name of attribute to retrieve
     * @return First value of attribute or '-'
     */
    protected String getValue( LDAPEntry entry,
                               String attrName ) {
        LDAPAttribute attr = entry.getAttribute( attrName );
        if ( attr == null ) {
            return "-";
        }
        Enumeration enumVals = attr.getStringValues();
        // Enumerate on values for this attribute
        boolean hasVals = false;
        if ( (enumVals == null) || 
             !enumVals.hasMoreElements() ) {
            return "-";
        }
    
        return (String)enumVals.nextElement();
    }       

    private LDAPConnection ld = null;
    private String host;
    private int port;
    private String conf;
    private String base;
    private String authdn;
    private String authpw;
    private int scope = LDAPConnection.SCOPE_SUB;
    private TextArea output;
    private TextField srch;
    private Button searchb;
    // Attributes to display for each entry found
    private static final String[] ATTRS =
        {"cn","mail","telephonenumber"};
    // Filter configuration file object
    private LDAPFilterDescriptor filterDesc = null;
}
