/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.util.*;
import netscape.ldap.*;

/**
 * Class that will move a specified entry to a new location,
 * including any subentries under it.
 * Subentries that are leaf nodes will have their original
 * entry deleted prior to the new one being created. This is
 * to avoid constraint violations against duplicate uids
 * existing in two places in the tree at the same time.
 */
public class MoveNode {

    /**
     * Move an entry or subtree
     *
     * @param args host, port, dn, new location, authdn,
     * password, YES | N
     */
    public static void main( String[] args ) {

        if ( args.length != 7 ) {
            System.out.println( "Usage: java MoveNode " +
                                "<host> <port> " +
                                "<dn> <newlocation> " +
                                "<authdn> <password> " +
                                "YES|N" );
            System.out.println( "Example:" );
            System.out.println( "    java MoveNode " +
                                "localhost 389 " +
                                "\"ou=people," +
                                "o=airius.com\" " +
                                "\"ou=people," +
                                "ou=employees," +
                                "o=airius.com\" " +
                                "\"cn=Directory Manager\" " +
                                "password YES" );
            System.out.println( "  will move " +
                                "ou=people,airius.com to " +
                                "ou=people,ou=employees," +
                                "airius.com");
            System.out.println( "YES|N is a confirmation " +
                                "for actually doing the " +
                                "deletion");
            System.out.println( "YES must be in all caps " +
                                "and spelled out!" );
            System.exit(1);
        }

        String host = args[0];
        int port = Integer.parseInt( args[1] );
        String branch = args[2];
        String newDN = args[3];
        String authid = args[4];
        String authpw = args[5];

        boolean doMove = false;
        if (args[6].equals("YES") ) {
            System.out.println( "Moving " + branch + " to " +
                                newDN + '!' );
            doMove = true;
        } else {
            System.out.println("Not doing the move");
        }

        LDAPConnection ld = new LDAPConnection();
        try {
            // Connect to server and authenticate
            ld.connect( host, port, authid, authpw );

            // Start recursively moving
            move( branch, newDN, ld, doMove );
            
            // If you want the old tree deleted, call the
            // BranchDelete.delete method here
            
        } catch( LDAPException e ) {
            System.out.println( e.toString() );
        }

        // Done, so disconnect
        if ( (ld != null) && ld.isConnected() ) {
            try {
                ld.disconnect();
            } catch ( LDAPException e ) {
                System.out.println( e.toString() );
            }
        }
        System.exit(0);
    }       

    /**
     * Move an entry or a subtree to a new location
     *
     * @param dn Entry or subtree to move
     * @parentDN New parent location
     * @param ld Active connection to directory
     * @param doMove <CODE>true</CODE> if the entries really
     * are to be moved
     * @exception LDAPException on failure
     */
    public static void move( String dn,
                             String parentDN, 
                             LDAPConnection ld,
                             boolean doMove )
                             throws LDAPException {
        // Read all attributes of the entry
        LDAPEntry entry = ld.read( dn );
        // Compose new DN of RDN plus new parent
        String[] rdns =
            LDAPDN.explodeDN( entry.getDN(), false );
        String newDN = rdns[0] + "," + parentDN;
        // Create entry object with new DN and old attributes
        LDAPEntry newEntry =
            new LDAPEntry( newDN, entry.getAttributeSet() );

        // See if there are any children
        LDAPSearchConstraints cons =
            ld.getSearchConstraints();
        cons.setBatchSize( 0 );  // Get all at once
        LDAPSearchResults res =
            ld.search( dn,
                       ld.SCOPE_ONE,
                       "objectclass=*",
                       ATTRS,
                       false,
                       cons );
        // Recurse on entries under this entry - if any
        System.out.println( "Copying DN:" + dn +
                            " to -> " + newDN );
        if ( !res.hasMoreElements() ) {
            // Single entry so delete original first to
            // avoid constraint violations (duplicate uid)
            if ( doMove ) {
                ld.delete( dn );
                try {
                    ld.add( newEntry );
                } catch ( LDAPException ldape ) {
                    // Error adding new entry so put old
                    // one back
                    System.out.println( "Could not add " +
                                        "new entry, " +
                                        "restoring old " +
                                        "one");
                    ld.add(entry); 
                }
            }
        } else {
            // This is a branch node so copy it
            if ( doMove ) {
                ld.add(newEntry);
            }
            while ( res.hasMoreElements() ) {
                // Recurse on moving all children
                try {
                    LDAPEntry theEntry = res.next();
                    String theDN = theEntry.getDN();
                    move( theDN, newDN, ld, doMove );
                } catch ( LDAPReferralException e ) {
                    // Do not follow referrals, just list
                    // them
                    System.out.println( "Search " +
                                        "reference: " );
                    LDAPUrl refUrls[] = e.getURLs();
                    for ( int i = 0; i < refUrls.length;
                          i++ ) {
                        System.out.println(
                            "  " + refUrls[i].getUrl() );
                    }
                    continue;
                }
            }
        }
    }

    private final static String[] ATTRS = {LDAPv3.NO_ATTRS};
}
