/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import netscape.ldap.*;

/**
 * Applet that displays a tree view of a directory
 */
public class DirBrowser extends JApplet {

    /**
     * Default constructor
     */
    public DirBrowser() {
        /* Netscape Communicator and Internet Explorer 4.0
           unconditionally print an error message to the
           Java console when an applet attempts to access
           the AWT system event queue. */
        getRootPane().putClientProperty(
            "defeatSystemEventQueueCheck",
            Boolean.TRUE);
    }

    /**
     * Constructor with explicit parameters, for calling
     * as an application
     *
     * @param host Host name of directory server
     * @param port Port number of directory server
     * @param base Base DN of DIT to display
     */
    public DirBrowser( String host, int port, String base ) {
        this.host = host;
        this.port = port;
        this.base = base;
        isApplet = false;
    }

    /**
     * Standard applet entry point; create the tree view
     */
    public void init() {
        /* Get parameters from applet tags if present */
        if ( isApplet ) {
            String s = getParameter( "host" );
            if ( (s != null) && (s.length() > 0) ) {
                host = s;
            }
            s = getParameter( "base" );
            if ( (s != null) && (s.length() > 0) ) {
                base = s;
            }
            s = getParameter( "port" );
            if ( (s != null) && (s.length() > 0) ) {
                port = Integer.parseInt( s );
            }
        }
        /* Create the root of the tree and
           get the entries beneath the root. */
        DefaultMutableTreeNode root = 
            new DefaultMutableTreeNode( base );
        addChildren( root );
        /* Set up the area for displaying attributes. */
        attributeListing = new JTextArea();
        attributeListing.setEditable(false);
        JScrollPane attributePane =
            new JScrollPane(attributeListing);
        attributePane.setVerticalScrollBarPolicy(
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        attributePane.setHorizontalScrollBarPolicy(
            JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        Dimension dim = new Dimension(300, 150);
        attributePane.setMinimumSize(dim);
        attributePane.setPreferredSize(dim);
        /* Create the tree for displaying entries
           in the directory hierarchy. */
        final JTree tree = new JTree( root );
        tree.getSelectionModel().setSelectionMode
            (TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.addTreeExpansionListener(
            new TreeExpansionListener() {
            /* If the tree is expaneded, get the 
               children of the expanded entry. */
            public void treeExpanded(TreeExpansionEvent e) {
                DefaultMutableTreeNode entry = 
                    (DefaultMutableTreeNode)
                    e.getPath().getLastPathComponent();
                addChildren(entry);
            }
            public void treeCollapsed(TreeExpansionEvent e) {
            }
        });
        tree.addTreeSelectionListener(
            new TreeSelectionListener() {
            /* If a node or leaf is selected, display 
               the attributes of that entry. */
            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode entry = 
                    (DefaultMutableTreeNode)
                    e.getNewLeadSelectionPath().
                    getLastPathComponent();
                String attributeString =
                    flattenAttributes(entry);
                attributeListing.setText(attributeString);
                attributeListing.setCaretPosition(0);
            }
        });
        JScrollPane treeView = new JScrollPane(tree);
        dim = new Dimension(300, 150);
        treeView.setMinimumSize(dim);
        treeView.setPreferredSize(dim);
        /* Set up the tree and textarea components
           in a split-pane. */
        JSplitPane twoPanes = 
            new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        twoPanes.setTopComponent(treeView);
        twoPanes.setBottomComponent(attributePane);
        twoPanes.setDividerLocation(150);
        /* Add the split-pane to the main content pane. */
        getContentPane().add(twoPanes, 
                             BorderLayout.CENTER);
    }
    
    /**
     * Display the entries one level
     * below a selected entry. The parent argument
     * identifies the node to be expanded.
     */
    protected void addChildren(
        DefaultMutableTreeNode parent ) {
        DefaultMutableTreeNode entry = null;
        DefaultMutableTreeNode subEntry = null;
        LDAPEntry childEntry, grandchildEntry;
        String childEntryDN;

        try {
            LDAPConnection conn = getConnection();
            if ( conn == null ) {
                return;
            }
            /* Only get the cn attribute. */
            String[] attrNames = { "cn" };
            /* If the node is the root node, query the 
               directory to get the child entries.*/
            if ( parent.isRoot() ) {
                /* Do a search of all entries that are
                   one level below the current entry.*/
                LDAPSearchResults results = 
                    conn.search( base,
                                 conn.SCOPE_ONE, 
                                 "(objectclass=*)",
                                 attrNames,
                                 false );
                /* Get the DN for each result, and use 
                    the DN as the string value of the node. */
                while ( results.hasMoreElements() ) {
                    childEntry = results.next();
                    entry = new DefaultMutableTreeNode(
                        childEntry.getDN());
                    /* Add the new node as a child of the
                        top-level node. */
                    parent.add(entry);
                    /* Search for entries that are one level
                       beneath the child entry to determine
                       if the child entry is a leaf or node.*/
                    LDAPSearchResults resUnderEntry = 
                        conn.search( childEntry.getDN(), 
                                     conn.SCOPE_ONE,
                                     "(objectclass=*)", 
                                     attrNames,
                                     false );
                    /* Get the DN of each result and add the DN
                        as a "grandchild" node under the child
                        node.*/
                    while (resUnderEntry.hasMoreElements()) {
                        grandchildEntry = resUnderEntry.next();
                        subEntry =
                            new DefaultMutableTreeNode(
                                grandchildEntry.getDN());
                        entry.add(subEntry);
                    }
                }
                /* If the node passed in is not the root node,
                   the child entries (in the tree) should
                   already exist (from the previous query). Use
                   the tree hierarchy to get the child entries,
                   and then query each child entry to get the 
                   "grandchild" entries. (This determines
                   whether or not the child entry is a leaf or
                   a node.) */
            } else {
                /* Get the child entries in the tree. */
                Enumeration childEntries = parent.children();
                while (childEntries.hasMoreElements()) {
                    entry = (DefaultMutableTreeNode)
                        childEntries.nextElement(); 
                    /* Each existing child entry in the tree 
                       contains the DN of that entry. Get the
                       DN. */
                    childEntryDN =
                        (String)entry.getUserObject();
                    /* Use this "child" DN to find the
                       "grandchild" entries. */
                    LDAPSearchResults resUnderEntry = 
                        conn.search( childEntryDN,
                                     conn.SCOPE_ONE, 
                                     "(objectclass=*)",
                                     attrNames,
                                     false);
                    /* Add each "grandchild" entry to the
                       tree.*/
                    while ( resUnderEntry.hasMoreElements() ) {
                        grandchildEntry = resUnderEntry.next();
                        subEntry =
                            new DefaultMutableTreeNode(
                                grandchildEntry.getDN());
                        entry.add( subEntry );
                    }
                }
            }
        } catch( Exception e ) {
            /* Any errors - connecting or searching - end up
               here and terminate the expansion */
            System.out.println( e.toString() );
        }  
    }

    /**
     * "Flatten" the list of attributes as a string. 
     * Instead of using LDAPEntry.toString(), which results in
     * a long string containing all attributes and values,
     * this method queries the directory for the entry and
     * returns a string containing newline-delimited attribute
     * names and values.
     */
    private String flattenAttributes(
        DefaultMutableTreeNode entry ) {
        String entryDN, attributeString;
        try {
            LDAPConnection conn = getConnection();
            if ( conn == null ) {
                return "";
            }
            /* Get the DN from the tree node. */
            entryDN = (String)entry.getUserObject();
            /* Get the corresponding LDAP entry from 
               the directory.*/
            LDAPEntry fullEntry = conn.read( entryDN );
            /* Get the attributes for the entry. */
            LDAPAttributeSet attributeSet =
                fullEntry.getAttributeSet();
            Enumeration enumAttrs =
                attributeSet.getAttributes();
            attributeString = "Attributes:";
            /* Build a string containing the attribute names
               and values. Delimit the names and values with
               newline characters.*/
            while ( enumAttrs.hasMoreElements() ) {
                LDAPAttribute anAttr =
                    (LDAPAttribute)enumAttrs.nextElement();
                String attrName = anAttr.getName();
                attributeString =
                    attributeString + "\n  " + attrName;
                Enumeration enumVals =
                    anAttr.getStringValues();
                while ( enumVals.hasMoreElements() ) {
                    String aVal =
                        (String)enumVals.nextElement();
                    attributeString =
                        attributeString + "\n    " + aVal;
                }
            }
            attributeString += "\n";
            return attributeString;
        } catch( Exception e ) {
            System.out.println(e.toString());
        }
        return "";
    }

    /**
     * Connect to server, if not already done
     *
     * @return Connection to directory server
     */
    protected LDAPConnection getConnection() {
        if ( connection == null ) {
            try {
                /* Connect to the LDAP server. */
                connection = new LDAPConnection();
                connection.connect( host, port );
            } catch ( LDAPException ex ) {
                System.out.println( ex.toString() );
                connection = null;
            }
        }
        return connection;
    }

    /**
     * Display the directory browser applet in a frame
     *
     * @param args host, port, baseDN
     */
    public static void main( String[] args ) {
        if ( args.length != 3 ) {
            System.out.println( "Usage: java DirBrowser " +
                                "<host> <port> <baseDN>" );
            System.exit( 1 );
        }
        String host = args[0];
        int port = Integer.parseInt( args[1] );
        String baseDN = args[2];
        DirBrowser browser = new DirBrowser( host,
                                             port,
                                             baseDN );
        browser.init();
        JFrame frame = new JFrame( "Directory Browser" );
        frame.addWindowListener( new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Window win = e.getWindow();
                win.setVisible(false);
                win.dispose();
                System.exit(0);
            }
        } );
        frame.getContentPane().add( browser );
        frame.pack();
        // Center frame
        Dimension screenSize =
            Toolkit.getDefaultToolkit().getScreenSize();
        Dimension size = frame.getSize();
        screenSize.height = screenSize.height/2;
        screenSize.width = screenSize.width/2;
        size.height = size.height/2;
        size.width = size.width/2;
        int y = screenSize.height - size.height;
        int x = screenSize.width - size.width;
        frame.setLocation(x, y );
        frame.show();
    }

    /* JTextArea for displaying attributes of an entry. */
    private JTextArea attributeListing;
    /* Connection to directory */
    private LDAPConnection connection = null;
    private String host = "localhost";
    private int port = 389;
    private String base = "o=airius.com";
    private boolean isApplet = true;
}
